/*
 * Created on Apr 7, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import eve.fx.Brush;
import eve.fx.Color;
import eve.fx.Graphics;
import eve.fx.Image;
import eve.fx.Pen;
import eve.fx.Point;
import eve.fx.Rect;
import eve.fx.gui.WindowSurface;
import eve.sys.Task;
import eve.ui.event.PenEvent;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
class HoldDownDisplay extends Task{

static HoldDownDisplay hd = new HoldDownDisplay();

HoldDownDisplay() 
{
	start();
}
static int cw = Control.isDoubleSized() ? 14 : 7;
static final int cp = 0;
static final int blockWidth = 5*(cw+cp)+2;
int id = 0;
int state = -1;
int windowX, windowY;
int startX, startY;
int pauseTime = 0;
Control target;
WindowSurface win;
Image captured, changing;
Graphics g;
long started;
boolean showIt;
synchronized void start(Control target,int x, int y,boolean repeatedHold)
{
	cancel();
	id++;
	state = 0;
	this.target = target;
	Window w = target.getWindow();
	win = w.surface;
	Point p = target.getPosInParent(w,null);
	startX = x+p.x;
	startY = y+p.y;
	windowX = startX-blockWidth/2;
	windowY = startY-blockWidth/2;
	showIt = !repeatedHold && target.showHoldDownIndicator(x,y) && !Gui.hasMousePointer;
	pauseTime = repeatedHold ? target.holdTick : target.holdDownPause;
	started = System.currentTimeMillis();
	notifyAll();
}
synchronized void cancel(int eventType, int x, int y)
{
	int res = 4;
	boolean c = eventType == PenEvent.PEN_UP;
	if (!c){
		int dx = startX-x;
		c = dx > res || dx < -res;
	}
	if (!c){
		int dy = startY-y;
		c = dy > res || dy < -res;
	}
	if (c) cancel();
}
synchronized void cancel()
{
	if (captured != null){
		win.drawImage(captured,windowX,windowY,blockWidth,blockWidth,windowX,windowY,blockWidth,blockWidth);
		captured.free();
		changing.free();
		g.free();
	}
	captured = changing = null;
	target = null;
	win = null;
	g = null;
	state = -1;
}
static final int ignoreState = 4;
static final int maxState = 8+ignoreState;
static final float c = 0.7f;
static final float[] xoffsets = {2,4-c,4,4-c,2,c,0,c}; 
static final float[] yoffsets = {0,c,2,4-c,4,4-c,2,c}; 
protected void doRun()
{
	try{
	while(!shouldStop) synchronized(this){
		//
		while(state == -1){
			try{
				wait();
			}catch(InterruptedException e){}
		}
		//
		if (state < maxState){
			if (showIt){
				if (state >= ignoreState){
					if (state == ignoreState){
						captured = win.getCompatibleImage(blockWidth,blockWidth);
						changing = win.getCompatibleImage(blockWidth,blockWidth);
						if (win.canCapture()){
							win.captureImage(captured,windowX,windowY,blockWidth,blockWidth);
						}else{
							Graphics g = new Graphics(captured);
							g.translate(-windowX,-windowY);
							target.getWindow().repaintNow(g,new Rect(windowX,windowY,blockWidth,blockWidth));
							g.free();
						}
						g = new Graphics(changing);
						g.drawImage(captured,0,0);
						g.changePen(Color.Black,Pen.SOLID,1);
						g.changeBrush(Color.LighterBlue,Brush.SOLID);
					}
					int x = (int)(xoffsets[state-ignoreState]*(cw+cp));
					int y = (int)(yoffsets[state-ignoreState]*(cw+cp));
					g.paintEllipse(x,y,cw,cw,true,true);
					win.drawImage(changing,windowX,windowY,blockWidth,blockWidth,windowX,windowY,blockWidth,blockWidth);
				}
				state++;
			}else
				state = maxState;
			try{
				long t = (started+(state*pauseTime)/maxState)-System.currentTimeMillis();
				if (t > 0) wait(t);
			}catch(Exception e){}
		}else{
			final Control t = target;
			cancel();
			new WindowThreadTask(){
				protected void doRun(){
					t.heldDown();
				}
			}.run(t,false);
			// Notify;
		}
	}
	}catch(Throwable t){
		t.printStackTrace();
	}
}
}
//####################################################
