package eve.ui;
import java.util.Iterator;

import eve.fx.gui.IKeys;
import eve.sys.Cache;
import eve.ui.event.ControlEvent;
import eve.ui.event.KeyEvent;
import eve.ui.event.MenuEvent;
import eve.ui.event.PenEvent;
/**
* A MenuBar holds a set of PullDownMenu buttons - each of which is associated with a
* Menu. You can listen to MenuEvents generated by the MenuBar to determine which item
* has been selected.
**/
//##################################################################
public class MenuBar extends Holder{
//##################################################################
/**
 * Set this to false if you don't want menu fading.
 */
public boolean fade = GuiStyle.fadeInMenus;
{
	modify(NoFocus,0);
}
private CellPanel menus = new CellPanel();
public MenuBar()
{
	addLast(menus).setControl(VFILL|WEST);
}
public void addMenu(PullDownMenu m)
{
	menus.addNext(m).setControl(VFILL|WEST);
	if (m.menuState != null) m.menuState.fade = fade;
	//m.addListener(this);
}
public PullDownMenu addMenu(String name)
{
	return addMenu(new Menu(),name);
}
public PullDownMenu addMenu(Menu m,String name)
{
	PullDownMenu pdm = new PullDownMenu(name,m);
	addMenu(pdm);
	return pdm;
}

//-------------------------------------------------------------------
private boolean showMenuFor(Control n)
//-------------------------------------------------------------------
{
	MenuState mm = n.menuState;
	if (mm != null) mm.fade = false;
	try{
		return n instanceof ButtonControl ? ((ButtonControl)n).startDropMenu(null) : n.tryStartMenu(null);
	}finally{
		if (mm != null) mm.fade = fade;
	}
}
//===================================================================
public boolean nextMenu(boolean isLeft)
//===================================================================
{
	Control prev = null;
	Iterator it = isLeft ? menus.getCachedChildren(true) : menus.getCachedChildren(false);
	try{
		while (it.hasNext()){
			Control c = (Control)it.next();
			if (c.menuIsActive()){
				Control n = (Control)it.next();
				if (n == null) return false;
				//Gui.freezeSIP(this);
				c.menuState.closeMenu();
				showMenuFor(n);
			}
		}
	}finally{
		Cache.put(it);
	}
	return false;
}

void closeMenus()
{
	for (Iterator it = getAllDescendants(false); it.hasNext();){
		Control c = (Control)it.next();
		if (c instanceof PullDownMenu){
			((PullDownMenu)c).deactivateMenu();
		}
	}
}

int numActive = 0;

protected void menuChanged(Menu m, boolean shown)
{
	if (shown) {
		numActive++;
		if (numActive == 1) setActiveMenus(true);
		//System.out.println(numActive);
	}else{
		numActive--;
		if (numActive == 0) setActiveMenus(false);
		//System.out.println(numActive);
	}
}
/*
void menusChanged(boolean shown)
{
	if (shown) {
		numActive++;
		if (numActive == 1) setActiveMenus(true);
		System.out.println(numActive);
	}else{
		numActive--;
		if (numActive == 0) setActiveMenus(false);
		System.out.println(numActive);
	}
	
}
*/
void setActiveMenus(boolean activeMode)
{
	for (Iterator it = getAllDescendants(false); it.hasNext();){
		Control c = (Control)it.next();
		if (c instanceof PullDownMenu){
			((PullDownMenu)c).setActiveMode(activeMode);
			if (activeMode) PenEvent.wantPenMoved(c,PenEvent.OVERRIDE_NO_MOUSE|PenEvent.WANT_PEN_MOVED_OUTSIDE,true);
			else PenEvent.wantPenMoved(c,0,true);
		}
	}
}
//===================================================================
public void onControlEvent(ControlEvent ev)
//===================================================================
{
	if (ev.type == ev.MENU_SHOWN){
		((Control)ev.target).getMenu().addListener(this);
	}else if (ev.type == ev.MENU_SHOWN){
	}else if (ev.type == MenuEvent.NEXT_MENU_LEFT || ev.type == MenuEvent.NEXT_MENU_RIGHT){
		nextMenu(ev.type == MenuEvent.NEXT_MENU_LEFT);
	}
	super.onControlEvent(ev);
}
//-------------------------------------------------------------------
protected boolean doHotKey(Control from,KeyEvent ev)
//-------------------------------------------------------------------
{
	Control first = null;
	if (ev.key == IKeys.MENU) {
		Iterator it = menus.getCachedChildren(false);
		try{
			for (; it.hasNext();){
				Control c = (Control)it.next();
				if (first == null) first = c;
				if (c.menuIsActive()){
					c.menuState.closeMenu();
					return true;
				}
			}
		}finally{
			Cache.put(it);
		}
		if (first != null) showMenuFor(first);
		return true;
	}
	return super.doHotKey(from,ev);
}

//##################################################################
}
//##################################################################

