package eve.ui;
import eve.fx.IImage;
import eve.fx.Insets;
import eve.fx.Sound;
import eve.fx.gui.IKeys;

//##################################################################
public class MessageBox extends Form{
//##################################################################

public IImage icon;
public int type;
public MessageArea message = new MessageArea("");
public static int beepSound = Sound.BEEP_ALERT;//MB_ICONEXCLAMATION;
public boolean doBeep = true;
public String messageText = "";
public int textAlignment = CENTER;
public boolean useScrollBars = false;
public boolean useTextMessage = false;
/**
 * If this is true then the MessageBox will not add any of it's
 * own controls. You must add your controls yourself.
 */
public boolean customMake = false;

/**
 * Add the Control to the MessageBox and set the customMake to true.
 * @param c the control to display.
 * @return the control c.
 */
public Control displayInstead(Control c)
{
	customMake = true;
	return addLast(c);
}
/**
* You can create this and set it up before displaying the message box,
* but you must also set the "icon" variable to be the icon you wish to display.
**/
public ImageControl iconDisplay;

protected CellPanel addTo;

/**
* This gets a CellPanel for the adding of additional controls before the icon/message
* area.
**/
//===================================================================
public CellPanel getBeforePanel()
//===================================================================
{
	if (addTo == null) addLast(addTo = new CellPanel());
	CellPanel cp = new CellPanel();
	addTo.addLast(cp);
	return cp;		
}
/**
* This gets a CellPanel for the adding of additional controls after the icon/message
* area.
**/
//===================================================================
public CellPanel getAfterPanel()
//===================================================================
{
	if (addTo == null) addLast(addTo = new CellPanel());
	CellPanel cp = new CellPanel();
	addLast(cp);
	return cp;		
}
//==================================================================
protected void checkButtons()
//==================================================================
{
	doButtons(type);
	if (!hasExitButton()) titleOK = new Button(close);
	super.checkButtons();
}

//==================================================================
public void make(boolean reMake)
//==================================================================
{
	if (made) return;
	if (!customMake){
		CellPanel p = new CellPanel(); 
		p.defaultTags.set(TAG_INSETS,new Insets(1,1,1,1));
		if (icon != null) {
			if (iconDisplay == null) iconDisplay = new ImageControl(icon);
			p.addNext(iconDisplay).setCell(DONTSTRETCH).setControl(DONTFILL|CENTER);
		}
		
		TextMessage tm = useTextMessage ? new TextMessage(messageText) : new TextMessage("");
		tm.alignment = textAlignment;
		message.alignment = textAlignment;
		Control c = useTextMessage ? tm : tm.tryMessageArea(this,null,messageText,message);
		p.addLast(useScrollBars ? new ScrollBarPanel((ScrollClient)c) : c,STRETCH,FILL|CENTER);//.setControl(DontFill,Center);
		message.text = messageText;
		if (addTo != null){
			addTo.addLast(p);
		}else
			addLast(p);
	}
	modifyAll(KeepSIP,0);
	super.make(reMake);
}
//===================================================================
public void shown()
//===================================================================
{
	super.shown();
	if (doBeep)
		if (beepSound != 0) Sound.beep(beepSound);
}
/**
 * Create a MessageBox with no title or message. Set the <b>title</b>
 * and <b>messageText</b> fields directly and use <b>Form.doButtons()</b>
 * method to add standard buttons (e.g. "Yes", "No", "OK", "Cancel") to
 * the MessageBox.
 * @deprecated use MessageBox(String title) instead.
 */
public MessageBox()
//==================================================================
{
	setHotKey(0,IKeys.ESCAPE);
	moveable = resizable = false;
	borderWidth = 0;
	borderStyle = BDR_NOBORDER;
	if (Gui.hasSoftKeys) windowFlagsToSet |= Window.FLAG_MAXIMIZE_ON_PDA;
	dontAddSIPFiller = true;
}
/**
 * Use this to create a MessageBox to which you will add all your own
 * Controls. Remember to call doButtons(int type) after adding your
 * controls to add the button bar.
 * @param title the title for the MessageBox.
 */
public MessageBox(String title)
{
	this();
	customMake = true;
	this.title = title;
}
/**
 * Create a MessageBox.
 * @param title The title for the MessageBox.
 * @param message The message text data to display. To display an icon set the <b>icon</b>
 * field directly.
 * @param type one of the MB_XXX values as defined in FormBase.
 */
public MessageBox(String title,String message,int type)
//==================================================================
{
	this();
	this.title = title;
	messageText = message;
	this.type = type;
	//resizable = true;
}
//===================================================================
public void doAction(int how){exit(IDCANCEL);}
//===================================================================
/*
//===================================================================
public static void main(String args[])
//===================================================================
{
	Application.startApplication(args);
	MessageBox mb = new MessageBox("Testing","This is testing an\n an animated image.",MBYESNOCANCEL);
	
	//mb.icon = new ewe.graphics.AnimatedIcon("0|250|1|250|2|250|3|250|4|250|5|250|6|250|7|250|8|250|9|250|10|1000","ewesoft/apps/jewel/build*.png",null);
	mb.execute();
	//mb.icon.free();
	System.exit(0);
}
*/
//##################################################################
}
//##################################################################

