/*
 * Created on Mar 22, 2007
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import java.util.Vector;

import eve.fx.Color;
import eve.fx.Dimension;
import eve.fx.IImage;
import eve.fx.ISurface;
import eve.fx.Point;
import eve.fx.Rect;
import eve.sys.Countdown;
import eve.ui.event.PenEvent;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
class PenHandler {
	
	SingleContainer theTip;
	Control theTipFrame;
	boolean softTip = false;
	
//	-------------------------------------------------------------------
	private void makeTip()
//	-------------------------------------------------------------------
	{
		if (theTip != null) return;
		theTip = new SingleContainer();//MessageArea("Hello");
		theTipFrame = theTip;//new Frame();
		//theTipFrame.contentsOnly = true;
		if (softTip){
			theTip.modify(Control.Transparent,0);
			theTip.borderWidth = 3;
			theTip.borderStyle = theTip.BDR_OUTLINE|theTip.BF_RECT|theTip.BF_SOFT|theTip.BF_MIDDLE;
		}else{
			theTip.borderWidth = 3;
			theTip.borderStyle = theTip.BDR_OUTLINE|theTip.BF_RECT;
		}
		theTip.backGround = new Color(255,255,200);
		theTip.foreGround = theTip.borderColor = Color.Black;
		theTip.modify(theTip.AlwaysRecalculateSizes,0);
	}
	
	private ToolTip currentTip = new ToolTip();
	private Control lastTipFor = null;
	//Control look = null;
	private Control lastMovedIn = null;
	private int lastCursor = 0;
	
	/**
	* 
	**/
//	===================================================================
	public synchronized boolean tipIsDisplayed() {return currentTip.forWho != null;}
//	===================================================================

	synchronized void setCursorFor(Window win, Control c, int cursor)
	{
		if (c == lastMovedIn){
			win.setCursor(cursor);
		}
	}
//	===================================================================
	private synchronized boolean refreshTip(ISurface surface)
//	===================================================================
	{
		if (currentTip.forWho == null) return false;
		try{
			if (currentTip.window == surface)
				theTipFrame.repaintNow();
		}catch(Exception e){}
		return true;
	}
	/*
//	===================================================================
	boolean lookingFor(Object what)
//	===================================================================
	{
		return what == look;
	}
*/
//	-------------------------------------------------------------------
	private static boolean onScreen(Control who)
//	-------------------------------------------------------------------
	{
		for (Control c = who; !(c instanceof Window); c = c.getParent())
			if (c == null) return false;
		return true;
	}
	
	private static Rect rbuff;
	
//	-------------------------------------------------------------------
	private static synchronized boolean isIn(PenEvent pe,Control who,int x,int y)
//	-------------------------------------------------------------------
	{
		if (pe.type == PenEvent.PEN_MOVED_OFF) return false;
		if (who.getWindow() != pe.window) return false;
		if ((PenEvent.getOptionsFor(who) & PenEvent.WHEN_NOT_ON_TOP_FRAME) == 0)
			if (!who.amOnTopFrame()) return false;
		if (rbuff == null) rbuff = new Rect();
		Rect r = Gui.getRectInWindow(who,rbuff,true);
		if (r == null) return false;
		if (r.width < 1 || r.height < 1) return false;
		if  (!(x >= r.x && x < r.x+r.width) || !(y >= r.y && y < r.y+r.height)) return false;
		if (!(who instanceof Container)) return true;
		Control ch = ((Container)who).findChild(x-r.x,y-r.y);
		if (ch != who && ch != null){
			return false;
		}else
			return true;
	}
//	===================================================================
	synchronized void resetCursor()
//	===================================================================
	{
		if (lastMovedIn == null) Gui.setCursor(0);
		else{
			int opts = PenEvent.getOptionsFor(lastMovedIn);
			int cr = opts >> 16;
			if (cr != 0) {
				Window w = lastMovedIn.getWindow();
				if (w != null) w.setCursor(cr);
				lastCursor = 0;
				//lastCursor = Gui.setCursor(lastMovedIn,cr);
			}
		}
	}
	/**
	* This is called by the VM when the WAIT_CURSOR has been removed.
	**/
//	===================================================================
	private void notWaiting()
//	===================================================================
	{
		resetCursor();
	}
//	===================================================================
	synchronized void topFrameChanging()
//	===================================================================
	{
		Control lmi = lastMovedIn;
		if (lastMovedIn == null) return;
		int opts = PenEvent.getOptionsFor(lastMovedIn);
		int cr = opts >> 16;
		if (cr != 0) {
			Window w = lastMovedIn.getWindow();
			if (w != null) w.setCursor(0);
			//Gui.setCursor(lastMovedIn,lastCursor);
		}
		PenEvent pe = new PenEvent();
		pe.type = PenEvent.PEN_MOVED_OFF;
		pe.target = lmi;
		lastMovedIn = null;
		Window win = lmi.getWindow();
		if (win != null) win.queueEvent(pe);
	}
	static Point lastPen = new Point();

//	##################################################################
	class tipTicker extends Countdown{
//	##################################################################
	{
		idleQuitTime = 5000;
	}
//	===================================================================
	public void ticked()
//	===================================================================
	{
		if (currentTip.forWho == null) lastTipFor = null;
	}

//	##################################################################
	}
//	##################################################################

	public boolean tipIsVisible = false;
	private ControlFader fader = null;
//	-------------------------------------------------------------------
	private void showTip(boolean immediate)
//	-------------------------------------------------------------------
	{
		showTip(immediate,null);	
	}
//	===================================================================
	private void showTip(boolean immediate,Object tooltip)
//	===================================================================
	{
		makeTip();
		if (lastMovedIn == null) return;
		Frame f = Gui.windowFrame(lastMovedIn);
		if (f == null) return;
		Object t = tooltip;
		{
			Point p = lastMovedIn.getPosInParent(f, Point.getCached(0,0));
			if (t == null) t = lastMovedIn.getToolTip(lastPen.x-p.x,lastPen.y-p.y);
			p.cache();
		}
		if (t == null) return;
		
		currentTip.forWho = lastMovedIn;
		if (t instanceof ToolTip){
			currentTip.persists = ((ToolTip)t).persists;
			t = currentTip.tip = ((ToolTip)t).tip;
		}else{
			Dimension dim = lastMovedIn.getSize(null);
			currentTip.persists = dim.height <= 32 || dim.width <= 32;
			currentTip.tip = t;
		}
		if (immediate & !currentTip.persists){
			currentTip.forWho = null;
			return;
		}
		if (t instanceof Control)
			theTip.setControl((Control)t);
		else if (t instanceof IImage){
			ImageControl ic = new ImageControl((IImage)t);
			ic.options = 0;
			theTip.setControl(ic);
		}else {
			Control c = new MessageArea(t.toString());
			theTip.setControl(c);
		}
		f.addDirectly(theTipFrame);
		currentTip.window = f.getWindow();
		
		theTipFrame.make(true);
		Dimension d = theTipFrame.getPreferredSize(null);
		Dimension fd = f.getSize(null);
		int y = lastPen.y+24;
		if (y+d.height > fd.height) y = lastPen.y-d.height-2;
		if (y < 0) y = 0;
		int x = lastPen.x-d.width/2;
		if (x+d.width > fd.width) x = fd.width-d.width-2;
		if (x < 0) x = 0;
		theTipFrame.setRect(x,y,d.width,d.height);
		if (immediate||true) theTipFrame.repaintNow();
		else{
			//fader = new ControlFader(theTipFrame);
			//fader.start();
		}
		theTipFrame.shown();
		tipIsVisible = true;
	}

//	-------------------------------------------------------------------
	private void requestHover()
//	-------------------------------------------------------------------
	{
		hoverCountdown.requestTick(500);
	}
	private void cancelHover()
	{
		hoverCountdown.cancelTick();
	}
	
	Countdown ticker = new tipTicker();

//	===================================================================
	void removeTip()
//	===================================================================
	{
		synchronized(hoverCountdown){
			if (fader != null) fader.cancel(false);
			tipIsVisible = false;
			if (currentTip.forWho != null){
				Frame f = Gui.windowFrame(theTipFrame);
				if (f != null){
					Point p = Gui.getPosInParent(theTipFrame,f,Point.getCached(0,0));
					Dimension tt = theTipFrame.getSize(Dimension.getCached(0,0));
					Rect r = Rect.getCached(p.x,p.y,tt.width,tt.height);
					theTipFrame.formClosing();
					f.remove(theTipFrame);
					f.repaintNow(null,r);
					p.cache();
					r.cache();
					tt.cache();
				}
				lastTipFor = currentTip.forWho;
				currentTip.forWho = null;
			}else{
				lastTipFor = null;
			}
			cancelHover();
		}
	}
	private boolean hoverLocked = false;

	private Countdown hoverCountdown = new Countdown(){
	{
		idleQuitTime = 5000;
	}
	protected void ticked(){
		showTip(false);
	}
	};
	/**
	 * This is called by the Window. You should not call it directly.
	 * @param win
	 * @param pe
	 * @param x
	 * @param y
	 * @return
	 */
//	-------------------------------------------------------------------
	boolean handlePenMove(Window win,PenEvent pe,int x,int y)
//	-------------------------------------------------------------------
	{
		if (pe.type == PenEvent.MOUSE_HOVER_OVER || pe.type == PenEvent.MOUSE_STOP_HOVER) return false;
		try{
			Control lmi = null;
			int ty = pe.type;
			boolean wasNotNull = false;
			synchronized(hoverCountdown){
				cancelHover();
				lmi = lastMovedIn;
				wasNotNull = lmi != null;
				if (lmi != null && !isIn(pe,lastMovedIn,x,y)){
					removeTip();
					ty = pe.type;
					pe.type = PenEvent.PEN_MOVED_OFF;
					ticker.requestTick(250);
					pe.target = lastMovedIn;
					pe.x = x; pe.y = y;
					int opts = PenEvent.getOptionsFor(lastMovedIn);
					//System.out.println("Setting off:"+lastCursor);
					win.setCursor(0);
					//Gui.setCursor(lastMovedIn,lastCursor);
					lastMovedIn = null;
				}else
					lmi = null;
			}
			if (lmi != null){
				win.dispatchPenEvent(pe);
				pe.type = ty;
				handlePenMove(win,pe,x,y);
				return true;
			}
			//
			// This does PEN_MOVED_ON
			//
			EventThreadData et = Gui.getEventThreadData();
			if (et.movedRefs == null) et.movedRefs = new Vector();
			Vector v = et.movedRefs;
			if (!wasNotNull){
				v.removeAllElements();
				PenEvent.getWantPenMoves(v);
				try{
					int sz = v.size();
					for (int i = 0; i<sz; i++){
						Control c = (Control)v.get(i);
						if (c == null) continue;
						int opts = PenEvent.getOptionsFor(c);
						if ((opts & PenEvent.WANT_PEN_MOVED_ONOFF) != 0){
							if (isIn(pe,c,x,y)){
								synchronized(hoverCountdown){
									ticker.cancelTick();
									lastMovedIn = c;
									lastPen.set(x,y);
								}
								ty = pe.type;
								pe.type = PenEvent.PEN_MOVED_ON;
								pe.target = lastMovedIn;
								pe.x = x; pe.y = y;
								win.dispatchPenEvent(pe);
								pe.type = ty;
								if (lastTipFor != null)
									if (lastTipFor.getParent() == c.getParent()) showTip(true);
									else requestHover();
								else requestHover();
								int cr = opts >> 16;
								if (cr != 0) {
									win.setCursor(cr);
									//lastCursor = Gui.setCursor(c,cr);
								}
								handlePenMove(win,pe,x,y);
								return true;
							}
						}
					}
				}finally{
					v.removeAllElements();
				}
			}
			// This does PEN_MOVE
			v.removeAllElements();
			PenEvent.getWantPenMoves(v);
			try{
				int sz = v.size();
				for (int i = 0; i<sz; i++){
					Control c = (Control)v.get(i);
					if (c == null || !onScreen(c)) continue;
					if (!onScreen(c)) continue;
					int opts = PenEvent.getOptionsFor(c);
					boolean in = isIn(pe,c,x,y);
					if (in){
						if (currentTip.forWho != null && !currentTip.persists)
							removeTip();
						if (currentTip.forWho == null){
							requestHover();
							lastPen.set(x,y);
						}
					}			
					if ((((opts & PenEvent.WANT_PEN_MOVED_INSIDE) != 0) && in) || ((opts & PenEvent.WANT_PEN_MOVED_OUTSIDE) != 0)){
						if (!c.isChildOf(win)) continue;
						pe.type = PenEvent.PEN_MOVE;
						pe.target = c;
						pe.x = x; pe.y = y;
						win.dispatchPenEvent(pe);
						//System.out.println(c);
					}
				}
			return false;
		}finally{
			v.removeAllElements();
		}
	}finally{
		
	}
}
}
//####################################################
