package eve.ui;

import eve.fx.Dimension;
import eve.fx.gui.SIPConstants;
import eve.sys.Vm;

/**
* A SipButton is a <b>placeholder</b> for the SipButton on a PocketPC system. You should
* never create a SipButton and add it to a Control. It is designed to be placed <b>either</b>
* in a CellPanel which will be added to the bottom of a Form (and thereby lie over the area
* where the SipButton would be) or to the extreme right of a mTabbedPanel (which should be
* set up with tabLocation being SOUTH) placed at the bottom of the Form.<p>
* 
<p>To use with the toolbar or tabbed panel use one of the static placeIn() methods.
**/

//##################################################################
public final class SIPButton extends Button{
//##################################################################

public static Dimension sipButtonSize = new Dimension(35,26);
static {
	initialize();
}

public static void initialize()
{
	int h = Vm.getParameter(Vm.GET_PARAMATER_SIP_BUTTON_HEIGHT);
	if (h > -1){
		//Device.tryMessageBox("SIP","Height: "+h,false);
		sipButtonSize.height = h;
		sipButtonSize.width = (h*35)/26;
	}
	sipButtonSize = new Dimension(35,26);	
}
static boolean sipActuallyOnRight = false; 
//-------------------------------------------------------------------
static boolean simulate()
//-------------------------------------------------------------------
{
	return (Vm.getParameter(Vm.SIMULATE_SIP) != 0);
}
private static boolean onBottom = true;
/*
static {
	if (simulate) Vm.setParameter(Vm.SIMULATE_SIP,1);
}
*/
/**
Simulate the SIP on a desktop system.
**/
//===================================================================
public static void simulateSIP()
//===================================================================
{
	Vm.setParameter(Vm.SIMULATE_SIP,1);
}
//-------------------------------------------------------------------
SIPButton()
//-------------------------------------------------------------------
{
	//if (!mApp.rotated) 	
	//else setPreferredSize(sipButtonSize.height,sipButtonSize.width);
	setPreferredSize(sipButtonSize.width,sipButtonSize.height);
	//backGround = new ewe.fx.Color(255,0,0);
	boolean sim = simulate();
	modify(NoFocus | (sim ? 0 : Disabled|Invisible),TakesKeyFocus);
	if (sim){
		image = loadImage("eve/pencilsmall.png");
		arrowDirection = Up;
	}
}

//===================================================================
public static boolean hasSipButton()
//===================================================================
{
	return (onBottom && simulate()) || ((Vm.getParameter(Vm.VM_FLAGS) & Vm.VM_FLAG_SIP_BUTTON_ON_SCREEN) != 0);
}
/**
 * Place a SipButton in an empty CellPanel that will be used
as a tool/menu bar along the bottom of the screen. If the platform sports
a SipSelectButton in the bottom right of the screen (currently only
PocketPC) then a SipButton will be created and placed on the
right of the toolbar. An empty CellPanel is added to the left of the
of the SipButton and this is then returned. You should then add
your tools to this.
 * @param destination The destination CellPanel that will act as the tool/menu bar.
 * @return The CellPanel that you should add your controls to.
 */
//===================================================================
public static CellPanel placeIn(CellPanel destination)
//===================================================================
{
	if (!hasSipButton() && !simulate()) return destination;
	if (Gui.hasSoftKeys) return destination;
	CellPanel cp = new CellPanel();
	destination.addNext(cp).setCell(HSTRETCH).setPreferredSize(1,1);
	destination.addNext(new SIPButton()).setCell(DONTSTRETCH);
	return cp;
}
/**
* Place in an mTabbedPanel display to the extreme right of the tabs. The mTabbedPanel should
* be set up with tabLocation being SOUTH.
* @param tp The mTabbedPanel to add the SipButton to.
* @return 
*/
//===================================================================
public static void placeIn(TabbedPanel tp)
//===================================================================
{
	if (!hasSipButton() && !simulate()) return;
	if (Gui.hasSoftKeys) return;
	tp.getExtremeControls(false).addNext(new SIPButton());
	tp.cardPanel.clearSipOnChange = true;
	tp.resizeForSIP();
}
//===================================================================
public void make(boolean reMake)
//===================================================================
{
	super.make(reMake);
	Window win = getWindow();
	if (win == null) {
		for (Control c = getParent(); c != null; c = c.getParent()){
			if (c instanceof FormFrame){
				Form f = ((FormFrame)c).myForm;
				f.windowFlagsToSet |= Window.FLAG_SHOW_SIP_BUTTON;
				return;
			}
		}
		return;
	}
	if ((win.getWindowFlags() & win.FLAG_SHOW_SIP_BUTTON) == 0){
		win.setWindowFlags(win.FLAG_SHOW_SIP_BUTTON);
		Gui.setSIP(win,2);
	}
}
public static boolean placeInSoftKeyBar(Panel sk)
{
	if (!Gui.hasSoftKeys || !hasSipButton()) return false;
	if (SoftKeyBar.preferredBarHeight < sipButtonSize.height)
		sk.setPreferredSize(-1, sipButtonSize.height);
	//if (!simulate()) return true;
	sk.addNext(new SIPButton()).setCell(VSTRETCH);
	return true;
}
//===================================================================
public void doAction(int how)
//===================================================================
{
	Window win = getWindow();
	Gui.setSIP(win,SIPConstants.SIP_OVERRIDE_USE_SIP|(Gui.getSIP(win) ^ SIPConstants.SIP_ON));
}
//##################################################################
}
//##################################################################

