/*
 * Created on Mar 10, 2006
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import eve.data.DateTimeUtils;
import eve.data.IDate;
import eve.data.PlainDate;
import eve.data.Value;
import eve.sys.Time;

/**
 * This allows a date to be entered as a String.
 *
 */
//####################################################
public class SimpleDateInput extends Input implements IDate{

	public String format = "dd/MM/yyyy";
	public String notEnteredText = "- Not Entered -";
	public String invalidText = "- Invalid Date -";
	private Time myTime = new Time();
	/**
	 * 
	 */
	public SimpleDateInput() {
		super();
	}

	protected void displayMyTime(boolean doInvalid)
	{
		if (myTime.getTime() == Time.invalidTime)
			text = doInvalid ? invalidText : notEnteredText;
		else{
			myTime.format = format;
			text = myTime.toString();
		}
		repaintDataNow();
	}
	public void setTime(Time t)
	{
		if (t == null) myTime.setTime(Time.invalidTime);
		else myTime.setTime(t.getTime());
		displayMyTime(false);
	}
	protected boolean textToTime(boolean doInvalid)
	{
		myTime.format = format;
		try{
			myTime.parse(text);
			displayMyTime(doInvalid);
			return true;
		}catch(IllegalArgumentException e){
			myTime.setTime(Time.invalidTime);
			displayMyTime(doInvalid);
			return false;
		}
	}
	public void lostFocus(int how)
	{
		textToTime(!notEnteredText.equals(text));
		super.lostFocus(how);
	}
	public Time getTime(Time dest)
	{
		if (dest == null) dest = new Time();
		dest.setTime(myTime.getTime());
		return dest;
	}
	public void setValue(Value data)
	{
		if (data instanceof Time) setTime((Time)data);
		else if (data instanceof IDate) DateTimeUtils.transferDate((IDate)data,this);
	}
	public void getValue(Value data)
	{
		if (data instanceof Time) getTime((Time)data);
		else if (data instanceof IDate) DateTimeUtils.transferDate(this,(IDate)data);
	}
	/*
	public void setValue(Value v)
	{
		setTime((Time)v);
	}
	public void getValue(Value v)
	{
		getTime((Time)v);
	}
	*/

	public boolean dateIsValid() {
		return myTime.dateIsValid();
	}

	public PlainDate getDate(PlainDate dest) {
		return myTime.getDate(dest);
	}

	public void makeDateInvalid() {
		myTime.makeDateInvalid();
	}

	public void setDate(int day, int month, int year) {
		myTime.setDate(day,month,year);
		displayMyTime(false);
	}
}

//####################################################
