/*
 * Created on Feb 28, 2006
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui;

import java.util.Vector;

import eve.fx.gui.RawEvent;
import eve.fx.gui.SurfaceCustomEvent;
import eve.fx.gui.SurfaceEvent;
import eve.fx.gui.SurfaceSIPEvent;
import eve.sys.Device;
import eve.sys.Event;
import eve.ui.event.ControlEvent;
import eve.ui.event.KeyEvent;
import eve.ui.event.PenEvent;

/**
 * @author Michael L Brereton
 *
 */
//####################################################
class WindowEventThread extends Thread{

Window window;
Object blockingOn;
private boolean closed, exited, bypassed;
private Object reusedEvents = RawEvent.createReusableEvents();
private WindowEventThread previous;
private Object rawEventLock;
private WindowRawEventThread raw;
private static Vector disabled;
private Control disabling;
//private EventBuffer eventBuffer;

//public static boolean noBuffering = true;
KeyEvent _keyEvent = new KeyEvent();
PenEvent _penEvent = new PenEvent();
SurfaceSIPEvent _sipEvent = new SurfaceSIPEvent();
ControlEvent _controlEvent = new ControlEvent();
static boolean _inPenDrag,_inMouseMove,_inMouseSize,_freeDragging;
boolean freezeSip;

EventThreadData data = new EventThreadData();

static synchronized boolean canDeliverTo(Control who)
{
	if (disabled == null) return true;
	int size = disabled.size();
	for (int i = 0; i<size; i++){
		Object c = disabled.get(i);
		if (who == c) return false;
		if (!(c instanceof Container)) continue;
		if (who.isChildOf((Container)c)) return false;
	}
	return true;
}

static Object execThread(Window forWho,Control toDisable)
{
	if (forWho == null || forWho.holdEvents) return null;
	synchronized(WindowEventThread.class){
		if (toDisable != null){
			if (disabled == null) disabled = new Vector();
			disabled.add(toDisable);
		}
	}
	return new WindowEventThread(forWho,toDisable);
}
static void endExecThread(Object execed)
{
	if (execed == null) return;
	WindowEventThread wet = (WindowEventThread)execed;
	wet.close(true);
	if (wet.disabling != null) synchronized(WindowEventThread.class){
		disabled.remove(wet.disabling);
		if (disabled.size() == 0) disabled = null;
	}
}
public void close()
{
	closed = true;
	synchronized(rawEventLock){
		rawEventLock.notifyAll();
	}
}
public void close(boolean waitForExit)
{
	close();
	if (!waitForExit) return;
	synchronized(rawEventLock){
		while(!exited)try{
			rawEventLock.wait();
		}catch(Exception e){}
	}
}

WindowEventThread(boolean dummy)
{
	if (dummy != true) throw new IllegalArgumentException();
}
WindowEventThread(Window forWho)
{
	this(forWho,null);
}
WindowEventThread(Window forWho,Control toDisable)
{
	//System.out.println(System.identityHashCode(this)+" for: "+System.identityHashCode(forWho));
	//Vm.debugStackTrace();
	disabling = toDisable;
	if (forWho == null){
		rawEventLock = this;
		exited = true;
	}else{
		//eventBuffer = forWho.eventBuffer;
		rawEventLock = forWho.rawEventThread;
		raw = forWho.rawEventThread;
		synchronized(rawEventLock){
			window = forWho;
			previous = window.activeThread;
			//System.out.println(this+" setting active thread: "+this);
			forWho.activeThread = this;
			rawEventLock.notifyAll();
		}
		start();
	}
}

public void run()
{
	//System.out.println("Thread is running: "+this);
	Device.setThreadIsEventHandler();
	try{
		// Inner loop, clearing events.
	while(true){
		synchronized(rawEventLock){
			//System.out.println(window+" +REL0");
			if (window.activeThread != this){
				//
				// Wait until you are the active thread.
				//
				try{
					rawEventLock.wait();
				}catch(InterruptedException e){}
				if (closed || raw.closed) break;
			}else{
				//
				// This is in case there are any left over events from the
				// previous thread.
				//
				//System.out.println("Clearing old!");
				raw.clearOldEvent();
				break;
			}
		}
	}
	//System.out.println(window+" -REL0");
			try{
				while(true){
					SurfaceEvent se = null;
					synchronized(rawEventLock){
						//System.out.println(this+" is? "+window.activeThread);
						if (window.activeThread == this) Gui.applySip();
						freezeSip = false;
						//System.out.println(window+" +REL1(2)");
						if (closed || raw.closed) break;
						if (window.activeThread != this || raw.gotEvent == null){
							try{
								//System.out.println(this+" ++REL1");
								rawEventLock.wait();
								//System.out.println(this+" --REL1");
							}catch(InterruptedException e){
								
							}
							continue;
						}
						//System.out.println("Yes, active thread: "+window.activeThread);
						if (closed || raw.closed) break;
						/*
						if (raw.gotEvent == null){
							try{
								//if (modalWindow != null) System.out.println("Waiting: "+se);
								rawEventLock.wait();
							}catch(InterruptedException e){
								
							}
							continue;
						}
						*/
						raw.gotEvent.inUse = true;
						raw.firstEvent = false;
						
						se = raw.gotEvent.convertToReusableSurfaceEvent(reusedEvents);
						if (se == null){
							System.out.println("Null event: "+raw.gotEvent.rawEventType+", "+raw.gotEvent.surfaceEventType);
						}else{//
							//System.out.println("OK event: "+raw.gotEvent.rawEventType+", "+raw.gotEvent.surfaceEventType);
							//if (se instanceof SurfacePointerEvent && se.type == SurfacePointerEvent.PEN_DOWN)
							//	System.out.println(System.identityHashCode(se)+" from: "+System.identityHashCode(raw.gotEvent)+", "+Thread.currentThread().hashCode());
						}
						raw.eventUsed = true;
						//if (se instanceof SurfacePointerEvent && se.type == SurfacePointerEvent.PEN_UP)
							//mThread.nap(1000);
					}
					//System.out.println(window+" -REL1");
					//
					// If I am here then I am the active thread and
					// gotEvent is not null.
					//
					
					try{
						//if (modalWindow != null && !handleModalEvent(se,modalWindow)) continue;
						//Window.this.onEvent(se);
						if (se instanceof SurfaceCustomEvent){
							Event ev = ((SurfaceCustomEvent)se).event;
							//System.out.println(ev.getClass()+", "+raw.posted+", "+(++raw.rxed));
							//if (ev.target != null) System.out.println(this+"->"+ev.target.getClass());
							window.handleSurfaceEvent(ev);
						}else 
							window.handleSurfaceEvent(se);
							//eventBuffer.onEvent(se);
						//window.handleSurfaceEvent(se);
						//if (modalWindow != null) System.out.println("Done onEvent: "+eventBuffer.hashCode());
					}finally{
						synchronized(rawEventLock){
							//System.out.println(window+" +REL2");
							if (window.activeThread == this){
								raw.clearOldEvent();
								//if (se instanceof SurfaceCustomEvent) System.out.println("Clearing custom!");
							}else{
								//System.out.println("Active is: "+window.activeThread);
							}
							//else System.out.println(this+" is not active thread: "+window.activeThread);
							//System.out.println(window+" -REL2");
						}
					}
				}
				//
				// Here I have broken out of the loop, so I am
				// in the process of exiting. At this point
				// I will deliver no more events.
			}finally{
				window.removeTip();
				synchronized(rawEventLock){
					exited = true;
					rawEventLock.notifyAll();
					while(window.activeThread != this && !bypassed){
						try{
							//System.out.println(window+" +REL3");
							rawEventLock.wait();
							//System.out.println(window+" -REL3");
						}catch(InterruptedException e){}
					}
					if (!bypassed){
						WindowEventThread wet = previous;
						boolean allowBypass = false;
						while (allowBypass && wet != null && wet.closed)
							wet = wet.previous;
						if (wet == null) wet = previous;
						//
						for (WindowEventThread w = previous; w != null && w != wet; w = w.previous)
							w.bypassed = true;
						//System.out.println(this+" returning active thread: "+wet);
						window.activeThread = wet;
						if (wet != null)
							synchronized(wet){
								if (wet.blockingOn != null)
									wet.interrupt();
							}
						rawEventLock.notifyAll();
						//System.out.println(System.identityHashCode(this)+" windowThread exiting!");
					}else{
						//System.out.println("Bypassed: "+this);
					}
				}
			}
	}finally{
		//System.out.println(System.identityHashCode(this)+" windowThread really exiting!");
	}
}
/*
protected void finalize()
{
	//System.out.println(System.identityHashCode(this)+" Finalize WET");
}
*/
}
//####################################################
