package eve.ui.data;
import java.util.Vector;

import eve.fx.Font;
import eve.fx.FontMetrics;
import eve.fx.IImage;
import eve.fx.IconAndText;
import eve.fx.Insets;
import eve.sys.Locale;
import eve.sys.Vm;
import eve.sys.Wrapper;
import eve.ui.Application;
import eve.ui.Button;
import eve.ui.ButtonCheckBox;
import eve.ui.CellPanel;
import eve.ui.Choice;
import eve.ui.ComboBox;
import eve.ui.FontInput;
import eve.ui.Gui;
import eve.ui.MenuItem;
import eve.ui.ScrollablePanel;
import eve.util.Utils;
import eve.util.mVector;

//##################################################################
public class FontChooser extends LiveObject{
//##################################################################

/**
* If this is set true, then the text in the drop-down menu for Fonts will
* be in the same Font instead of being rendered in their own face.
**/
public static boolean dontShowDifferentFonts = true;

public static String [] fonts;
protected static Vector fontItems;
public int size = 14;
public String font = "Helvetica";
public boolean bold = false;
public boolean italic = false;
public boolean underline = false;
public boolean chooseStyle = false;
public boolean chooseSize = true;
public boolean chooseName = true;
//public String _fields = "size,font,bold,italic,chooseStyle,underline";

//===================================================================
public FontChooser() {this(true);}
//===================================================================

//===================================================================
public FontChooser(boolean chooseStyle)
//===================================================================
{
	this.chooseStyle = chooseStyle;
}
//===================================================================
public boolean _getSetField(String fieldName, Wrapper wrapper, boolean isGet)
//===================================================================
{
	if (fieldName.equals("theFont")){
		if (isGet) wrapper.setObject(toFont());
		else fromFont((Font)wrapper.getObject());
		return true;
	}else
		return super._getSetField(fieldName,wrapper,isGet);
}

//===================================================================
protected ScrollablePanel getEditorScroller(CellPanel cp)
//===================================================================
{
	return null;
}
//===================================================================
public void addToPanel(CellPanel cp,Editor ed,int which)
//===================================================================
{
	ed.makeSmall();
	if (Gui.isSmartPhone){
		int options = 0;
		if (!chooseStyle) options |= FontInput.OPTION_NO_STYLE;
		if (!chooseSize) options |= FontInput.OPTION_NO_SIZE;
		if (!chooseName) options |= FontInput.OPTION_NO_NAME;
		ed.addField(cp.addLast(new FontInput(options)),"theFont$Leve/fx/Font;");
		return;
	}

	if (fonts == null){
		fonts = Font.listFonts(Application.mainApp.getSurface());
		Utils.sort(fonts,Vm.getLocale().getStringComparer(Locale.IGNORE_CASE),false);
		fontItems = new mVector(fonts);
		//
		if (!dontShowDifferentFonts){
			fontItems.clear();
			Font gf = Application.guiFont;
			int size = gf.getSize();
			for (int i = 0; i<fonts.length; i++){
				Font f = new Font(fonts[i],Font.PLAIN,size);
				FontMetrics fm = Application.mainApp.getFontMetrics(f);
				IImage ln = new IconAndText((IImage)null,fonts[i],fm);//fonts[i],fm);
				fontItems.add(new MenuItem(fonts[i],ln));
			}
		}
	}
	ComboBox mcb = new ComboBox();
	mcb.input.columns = 20;
	mcb.input.prompt = "Font Name";
	mcb.choice.items = fontItems;//new ewe.util.Vector(fonts);
	mcb.choice.menuOptions |= Choice.MENU_FULL_WIDTH;
	ed.addField(cp.addNext(mcb),"font").setCell(cp.HSTRETCH);
	if (chooseSize){
		mcb = new ComboBox(new String[]{"8","10","11","12","14","16","20"},0);
		mcb.input.columns = 3;
		mcb.input.prompt = "Font Size";
		ed.addField(cp.addNext(mcb),"size").setCell(cp.DONTSTRETCH);
	}
	if (chooseStyle){
		CellPanel style = new CellPanel(); style.equalWidths = true;
		ed.addField(style.addNext(new ButtonCheckBox("B")),"bold");
		ed.addField(style.addNext(new ButtonCheckBox("I")),"italic");
		ed.addField(style.addNext(new ButtonCheckBox("U")),"underline");
		cp.addNext(style).setCell(cp.DONTSTRETCH).setTag(cp.TAG_INSETS,new Insets(0,4,0,0));
		style.modifyAll(ed.NoFocus,0);
	}
	if (chooseSize){
		CellPanel sz = new CellPanel(); sz.equalWidths = true;
		ed.addField(sz.addNext(new Button("+")),"increase");
		ed.addField(sz.addNext(new Button("-")),"decrease");
		cp.addNext(sz).setCell(cp.DONTSTRETCH).setTag(cp.TAG_INSETS,new Insets(0,4,0,0));
		sz.modifyAll(ed.NoFocus,0);
	}
}
//===================================================================
public void fromFont(Font f)
//===================================================================
{
	if (f == null) return;
	int style = f.getStyle();
	bold = (style & Font.BOLD) != 0;
	italic = (style & Font.ITALIC) != 0;
	underline = (style & Font.UNDERLINE) != 0;
	font = f.getName();
	size = f.getSize();
}
//===================================================================
public Font toFont()
//===================================================================
{
	int style = Font.PLAIN;
	if (bold) style |= Font.BOLD;
	if (italic) style |= Font.ITALIC;
	if (underline) style |= Font.UNDERLINE;
	return new Font(font,style,size);
}
//===================================================================
public void action(String fieldName,Editor ed)
//===================================================================
{
	if (fieldName.equals("increase")){
		size++;
	}else if (fieldName.equals("decrease")){
		size--;
	}else {
		super.action(fieldName,ed);
	}
	ed.toControls("size");
	ed.notifyDataChange();
}
//===================================================================
public void fieldChanged(String fieldName,Editor ed)
//===================================================================
{
/*
	if (fieldName.equals("font") && !dontShowDifferentFonts){
			ComboBox mc  = (ComboBox)ed.findFieldTransfer(fieldName).dataInterface;
			mc.input.font = new Font(font,Font.PLAIN,size);
			mc.input.repaintNow();
	}
*/
	super.fieldChanged(fieldName,ed);
}
//##################################################################
}
//##################################################################

