package eve.ui.data;
import eve.ui.Control;
import eve.ui.Frame;
import eve.ui.Gui;
import eve.ui.Input;
import eve.ui.MessageBox;
import eve.ui.Window;
import eve.ui.event.ControlEvent;

/**
* This class can be used to accept a single line of input from the user.
* It will also be extended to be easily modified to accept more types of data.
**/
//##################################################################
public class InputBox extends Editor{
//##################################################################
{
	windowFlagsToSet &= ~Window.FLAG_MAXIMIZE_ON_PDA;
}

//===================================================================
public InputBox()
//===================================================================
{
	Gui.setOKCancel(this);
}

//===================================================================
public InputBox(String title)
//===================================================================
{
	this();
	this.title = title;
}


Control singleInput;
//===================================================================
public String input(String initialValue,int width)
//===================================================================
{
	return input(null,initialValue,width,false);
}
//===================================================================
public String input(Frame parent,String initialValue,int width)
//===================================================================
{
	return input(parent,initialValue,width,false);
}
//===================================================================
public String input(Frame parent,String initialValue,int width,boolean isPassword)
//===================================================================
{
	singleInput = new Input(initialValue);
	singleInput.columns = width;
	((Input)singleInput).wantReturn = true;
	((Input)singleInput).isPassword = isPassword;
	addLast(singleInput).setCell(HSTRETCH);
	if (execute(parent,Gui.CENTER_FRAME) == IDCANCEL) return null;
	return singleInput.getText();
}

//-------------------------------------------------------------------
protected boolean isAnInput(Object eventTarget)
//-------------------------------------------------------------------
{
	if (singleInput != null && eventTarget == singleInput) return true;
	return false;
}

//===================================================================
public void onControlEvent(ControlEvent ev)
//===================================================================
{
	if (isAnInput(ev.target)){
		if (ev.type == ev.PRESSED)
			exit(IDOK);
		else if (ev.type == ev.CANCELLED)
			exit(IDCANCEL);
	}
	else super.onControlEvent(ev);
}


/**
 * Use this if you wish from the isValid() method to display a message if the input is invalid.
 * @param title the title of the message box.
 * @param message the text of the message.
 */
//-------------------------------------------------------------------
protected void showErrorMessage(String title, String message)
//-------------------------------------------------------------------
{
	new MessageBox(title,message,MessageBox.MBOK).execute();
}
//-------------------------------------------------------------------
protected boolean canExit(int exitCode)
//-------------------------------------------------------------------
{
	if (exitCode == IDOK) 
		if (isValid(singleInput.getText())) return true;
		else{
			Gui.takeFocus(singleInput,ByRequest);
			return false;
		}
	return super.canExit(exitCode);
}
/**
 * Override this to validate the entered data before exit. If it returns false
 * the input box will not close.
 * @param enteredValue the value that was entered.
 * @return true if the data is valid, false if not.
 */
//-------------------------------------------------------------------
protected boolean isValid(String enteredValue)
//-------------------------------------------------------------------
{
	return true;
}
/**
 * Use this to get the entered text.
 * @return the entered text.
 */
//-------------------------------------------------------------------
protected String getInputValue()
//-------------------------------------------------------------------
{
	return singleInput.getText();
}
/**
 * Use this to set the entered text.
 * @return the entered text.
 */
//-------------------------------------------------------------------
protected void setInputValue(String text)
//-------------------------------------------------------------------
{
	singleInput.setText(text);
}
//##################################################################
}
//##################################################################

