package eve.ui.data;
import eve.data.DataUtils;
import eve.data.EncodableObject;
import eve.fx.IImage;
import eve.sys.Convert;
import eve.sys.FieldTransfer;
import eve.sys.Reflection;
import eve.ui.Application;
import eve.ui.CellPanel;
import eve.ui.Form;
import eve.ui.Gui;
import eve.ui.MenuItem;
import eve.ui.MessageBox;
import eve.ui.ScrollBarPanel;
import eve.ui.ScrollableHolder;
import eve.ui.ScrollablePanel;
import eve.ui.VerticalScrollPanel;
/**
* This is a complete implementation of LiveData which you can use as a
* base for creating new LiveData Objects. 
**/
//##################################################################
public class LiveObject extends EncodableObject implements LiveData{
//##################################################################
/**
* By default this returns the String "unnamed".
**/
//===================================================================
public String getName() {return "unnamed";}
//===================================================================

/**
 * By default this does a direct comparison between name and getName(). 
 * @see eve.data.Named#isNamed(java.lang.String)
 */
public boolean isNamed(String name)
{
	String me = getName();
	if (me == null) return name == null;
	else return me.equals(name);
}
/**
* This shoudl create the new Editor object but not add anything to it. This is called
* by getEditor(). By default it returns a standard Editor object.
**/
//-------------------------------------------------------------------
protected Editor makeNewEditor(int options)
//-------------------------------------------------------------------
{
	//return new AppForm();
	return new Editor();
}
/**
This is called by the default getEditor() should return a ScrollBarPanel for the CellPanel that the editor
is being placed in. By default, this is a ScrollBarPanel except on a
SmartPhone, where it will be a VerticalScrollPanel.<p>
You may return null from this. 
**/
//-------------------------------------------------------------------
protected ScrollablePanel getEditorScroller(CellPanel cp)
//-------------------------------------------------------------------
{
	ScrollablePanel sp = (Gui.isSmartPhone||Gui.isPDA) ? new VerticalScrollPanel(new ScrollableHolder(cp)) : new ScrollBarPanel(new ScrollableHolder(cp));
	sp.setClientConstraints(cp.HEXPAND|cp.HCONTRACT|cp.VEXPAND|cp.VCONTRACT);
	//sp.shrinkComponent = true;
	return sp;
}
/**
* This creates a new Editor Object - you should not override this method,
* rather you should override the addToPanel() method to provide a custom
* Editor for your LiveData.
**/
//===================================================================
public Editor getEditor(int options)
//===================================================================
{
	Editor ed = makeNewEditor(options);
	ed.objectClass = getClass();
	ed.sampleObject = this;
	CellPanel cp = new CellPanel();
	ScrollablePanel sp = getEditorScroller(cp);
	if (sp != null) ed.getProperties().set("MainScrollBarPanel",sp);
	CellPanel cp2 = (CellPanel)ed.getProperties().getValue("EditorContents",null);
	if (cp2 == null) ed.addLast(sp == null ? cp : sp);
	else cp2.addLast(sp == null ? cp : sp);
	ed.createSipFiller(cp2 == null ? ed : cp2);
	addToPanel(cp,ed,options);
	ed.setObject(this);
	return ed;
}
/**
 * Enable/Disable an Editors main scrollbar panel.<p>
 * By default, the getEditor() method will place a CellPanel into a ScrollBarPanel and then add it the Editor
 * before calling addToPanel(). However initially this ScrollBarPanel is disabled (i.e. it's "shrinkComponent" variable
 * is true - which results in ScrollBars never appearing). This method is used to enable it.
 * @param ed The editor created by getEditor().
 * @param enable true to enable scrolling, false to disable it.
 * @deprecated use Form.enableScrolling(boolean horizontally, boolean vertically) instead.
 */
//===================================================================
public static void enableEditorScrolling(Editor ed,boolean enable)
//===================================================================
{
	if (ed != null) ed.enableScrolling(enable);
}
/**
* This gets called by action(FieldTransfer ft,Editor ed) and by fieldChanged(FieldTransfer ft,Editor ed)
* if the field changed is a menu item.
* By default it will look for
* a method called void <fieldName>_action(Editor ed) and if it finds it, it will be executed.
**/
//===================================================================
public void action(String fieldName,Editor ed)
//===================================================================
{
	Editor.invokeFieldHandlerMethod(this,fieldName,ed,"action");
}
/**
* This gets called by fieldChanged(FieldTransfer ft,Editor ed). 
* By default it will look for
* a method called void <fieldName>_changed(Editor ed) and if it finds it, it will be executed.
**/
//===================================================================
public void fieldChanged(String fieldName,Editor ed)
//===================================================================
{
	Editor.invokeFieldHandlerMethod(this,fieldName,ed,"changed");
}
/**
* This implements the action() method in FieldListener. By default it
* calls action(String name,Editor ed)
* @param ft The FieldTransfer object representing the changed field.
* @param ed The Editor from which the event came.
**/
//===================================================================
public void action(FieldTransfer ft,Editor ed) {action(ft.fieldName,ed);}
//===================================================================
/**
* This implements the fieldChanged() method in FieldListener. It operates like
* like this:<p>
* If the field is of type ewe.ui.MenuItem then the method action(String name,Editor ed)
* is called with the "name" parameter being set to the "action" member of the selected MenuItem.
* <p>Otherwise the fieldChanged(String fieldName,Editor ed) method is called with the fieldName
* of the FieldTransfer object.
* @param ft The FieldTransfer object representing the changed field.
* @param ed The Editor from which the event came.
*/
//===================================================================
public void fieldChanged(FieldTransfer ft,Editor ed)
//===================================================================
{
	if (MenuItem.class.equals(ft.fieldType)){
		Object got = ft.getFieldValue(ed.myObject);
		if (got instanceof MenuItem){
			action(((MenuItem)got).action,ed);
		}
	}else
		fieldChanged(ft.fieldName,ed);
}
/**
* This implements the fieldEvent() method in FieldListener. 
**/
//===================================================================
public void fieldEvent(FieldTransfer ft,Editor ed,Object event)
//===================================================================
{
	if (event instanceof EditorEvent){
		EditorEvent ev = (EditorEvent)event;
		if (ev.type == EditorEvent.OBJECT_SET){
			if (ed.myObject == this){
				setToEditor(ed);
			}
		}
	}
}
/**
 * This is called when this Object is set as the object the Editor is now editing.
 * @param ed the active Editor. By default this does nothing.
 */
protected void setToEditor(Editor ed)
{
	
}
/**
* Override this to provide the user interface for editing the Object.
* Add all your controls to the CellPanel cp, and your fields to the Editor ed.
* The options parameter may be used to specify a different Editor for the
* Object for different situations - it is the same parameter that is passed
* to the LiveData.getEditor(int options) call.
**/
//===================================================================
public void addToPanel(CellPanel cp,Editor ed,int options)
//===================================================================
{
	//Vector v = DataUtils.getClassList(getClass(),"addToPanel(Leve/ui/CellPanel;Leve/ui/data/Editor;I)V");
	Class[] v = DataUtils.getCachedClassList(getClass(),"addToPanel(Leve/ui/CellPanel;Leve/ui/data/Editor;I)V");
	if (v.length != 0){
		UIBuilder u = UIBuilder.newInputStack(cp,ed,this);
		for (int i = 0; i<v.length; i++){
			String fields = DataUtils.getFieldList(v[i],this,true);
			if (fields.length() != 0){
				u.setFormats((String)getDeclaredFieldValue("_formats",(v[i]).getName()));
				u.addAll(fields);
			}
		}
		u.close(true);
	}
}
/**
* Use this within addToPanel() if you still want it's default of creating
* a new UIBuilder and adding an InputStack. It will also add in the fields for this object if needed.
* @param cp The CellPanel passed to addToPanel()
* @param ed The Editor passed to addToPanel()
* @param baseClassName The base class being added.
* @return The UIBuilder for the Editor with an open InputStack.
* @exception IllegalArgumentException if the baseClassName is not valid for this object.
*/
//-------------------------------------------------------------------
protected UIBuilder addMeToPanel(CellPanel cp,Editor ed,String baseClassName) throws IllegalArgumentException
//-------------------------------------------------------------------
{
	UIBuilder b = UIBuilder.newInputStack(cp,ed,this);
	if (baseClassName != null) b.setFormats((String)getDeclaredFieldValue("_formats",baseClassName)).addAll(getMyFieldList(baseClassName));
	return b;
}
/**
* This returns null by default.
**/
//===================================================================
public IImage getIcon() {return null;}
//===================================================================
/**
* This opens an Editor for this object.
* @param title The title for the editor. Can be null to leave as is.
* @param editorOption Options to pass to getEditor().
* @return true if the user pressed OK, false if the user pressed cancel.
*/
//===================================================================
public boolean input(String title,int editorOption)
//===================================================================
{
	Editor ed = getEditor(editorOption);
	if (title != null) ed.title = title;
	Gui.setOKCancel(ed);
	return (ed.execute() != ed.IDCANCEL);
}
/**
* This works on any object. It adds the fields for an object into a CellPanel that itself is
* added to the editor (or which itself may be the Editor).
* @param cp A CellPanel to add the object fields to.
* @param ed The Editor to be used to edit the object.
* @param objectOrClass The object or the Class/Reflect of the object.
* @param declaredOnly if this is true then only the declared fields will be added
* and not the fields of the superclass.
* @return the UIBuilder that was used to add the fields. The builder will have an open
* InputStack which contains the fields added.
* @exception IllegalArgumentException 
*/

//===================================================================
public static UIBuilder addObjectToPanel(CellPanel cp,Editor ed,Object objectOrClass,boolean declaredOnly) throws IllegalArgumentException
//===================================================================
{
	UIBuilder b = UIBuilder.newInputStack(cp,ed,objectOrClass);
	Object data = Reflection.toObject(objectOrClass);
	if (data != null) b.setFormats(DataUtils.appendAllFields("_formats",data,declaredOnly));
	String s = DataUtils.getFieldList(Reflection.toClass(objectOrClass),data,declaredOnly);
	b.addAll(s);
	return b;
}

/**
Get a Form to run this LiveObject as a stand-alone application.
**/
//===================================================================
public Form runAsApp()
//===================================================================
{
	Editor ed = getEditor(0);
	if (ed != null) {
		ed.enableScrolling(true,true);
		ed.exitSystemOnClose = true;
	}
	return ed;
}

//===================================================================
public static void main(String args[]) throws Exception
//===================================================================
{
	Application.startApplication(args);
	Class c = Class.forName(args[0]);
	LiveData ld = (LiveData)c.newInstance();
	int which = 0;
	if (args.length > 1) which = Convert.toInt(args[1]);
	Editor ed = ld.getEditor(which);
	ed.title = c.getName();
	ed.modifyAll(Editor.KeepSIP,0);
	ed.execute();
	System.exit(0);
}

public static void liveMain(String liveClass, String args[])
{
	Application.startApplication(args);
	Class c = Reflection.forName(liveClass);
	LiveData f = c != null ? (LiveData)Reflection.newInstance(c) : null; 
	if (f == null){
		new MessageBox("LiveData Error","Cannot create or execute: "+liveClass,Form.MBOK).execute();
		System.exit(-1);
	}
	Editor ed = f.getEditor(0);
	System.exit(ed.execute());
}
//##################################################################
}
//##################################################################

