package eve.ui.launcher;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import eve.data.Property;
import eve.data.PropertyList;
import eve.fx.IImage;
import eve.fx.Picture;
import eve.io.File;
import eve.io.FileRandomStream;
import eve.io.RandomStream;
import eve.io.StreamUtils;
import eve.io.filestore.EveFile;
import eve.io.filestore.FileStore;
import eve.io.filestore.FileStoreEntry;
import eve.server.RemoteAppEntry;
import eve.sys.ClassFile;
import eve.sys.Device;
import eve.ui.Button;
import eve.ui.CellPanel;
import eve.ui.Choice;
import eve.ui.Control;
import eve.ui.Frame;
import eve.ui.Gui;
import eve.ui.InputStack;
import eve.ui.MessageBox;
import eve.ui.ProgressBarForm;
import eve.ui.ReportException;
import eve.ui.VMOptions;
import eve.ui.data.Editor;
import eve.ui.data.LiveObject;
import eve.ui.data.UIBuilder;
import eve.util.FormattedDataSource;
import eve.util.TextDecoder;
import eve.util.TextEncoder;
import eve.util.mString;
import eve.util.mVector;
import eve.zipfile.ZipFile;
//##################################################################
public class VMApp extends LiveObject{
//##################################################################

public String name = "";
public String args = "";
public String target = "";
public String programDir = "";
public String category = "Eve";
public boolean copyOver = false;
public boolean execWithEve = true;
public IImage image;
public String location = "";
public int install;
public String startClass = "";
public String classPaths = "";
public String vmArgs = "";
public String imageName = "";

PropertyList all = null;
public String _fields = "startClass,name,args,target,programDir,execWithEve,install,vmArgs,imageName,classPaths";

//-------------------------------------------------------------------
protected TextEncoder encode(TextEncoder te)
//-------------------------------------------------------------------
{
	encodeFields(_fields,te,"VMApp");
	return super.encode(te);
}
void deleting()
{
	try{
		if (imageName != null)
			VMApps.getImageDirectory().getChild(imageName).delete();
	}catch(Exception e){}
}
//-------------------------------------------------------------------
protected TextDecoder decode(TextDecoder te)
//-------------------------------------------------------------------
{
	image = null;
	decodeFields(_fields,te,"VMApp");
	if (imageName != null && imageName.length() != 0){
		try{
			image = new Picture(new FormattedDataSource()
				.set(VMApps.getImageDirectory().getChild(imageName)),0);
		}catch(Exception e){
		}
	}else{
		//ewe.sys.Vm.debug("No image bytes found!");
	}
	/** Don't load - except when first using it - or possibly when installing.
	if (true) 
		ewe.sys.Vm.debug("Not doing Load!");
	else
		load();
	**/
	return super.decode(te);
}
public void copyFrom(Object other)
{
	if (other instanceof VMApp)
		all = ((VMApp)other).all;
	super.copyFrom(other);
}
public String baseName()
{
	String ret = null;
	if (all != null)
		if (install < all.size()){
			ret = ((Property)all.get(install)).name;
			if (ret.equals("default")) ret = null;
		}		
	if (ret == null && target.toLowerCase().endsWith(".eve")){
		ret = File.getNewFile(target).getName();
		ret = ret.substring(0,ret.length()-4);
	}
	if (ret == null){
		String start = target;
		if (start == null || start.length() == 0) 
			start = startClass;
		ret = mString.rightOf(start,'.');
		if (ret.length() == 0) ret = start;
	}
	if (ret != null)
		ret = ret.replace(' ','_');
	return ret;
}
public PropertyList chosenProperties()
{
	if (all == null) return null;
	if (all.size() == 0) return null;
	if (install >= all.size()) install = 0;
	return (PropertyList) ((Property)all.get(install)).value;
}
/**
An option for getEditor().
**/
public static final int ZAURUS_INSTALL = 1;
public static final int ADD_NEW = 2;
//===================================================================
public void addToPanel(CellPanel cp,Editor ed,int options)
//===================================================================
{
	ed.title = "Add Eve App";
	if (Gui.isSmartPhone || Gui.isPDA) ed.enableScrolling(true,true);
	UIBuilder b = new UIBuilder(ed,this,cp);
	boolean selectOnTop = (Gui.hasSoftKeys || Gui.hasSoftKeyboard);
	if (options == ADD_NEW && selectOnTop) {
		CellPanel cpp = b.open();
			cpp.addLast(ed.addField(new Button("Select App File"),"select"),Control.HSTRETCH,Control.DONTFILL);
		b.close(true).setCell(cp.HSTRETCH);
	}
	InputStack is = b.openInputStack();
	is.doubleLined = Gui.isSmartPhone || Gui.isPDA;
	is.inputLength = 30;
	int len = all == null ? 1 : all.size();
	if (len > 1){
		String [] ch = new String[len];
		for (int i = 0; i<len; i++) ch[i] = ((Property)all.get(i)).name;
		b.add("Install:","install",new Choice(ch,0)); 
	}
	if (options == 0 || options == ADD_NEW){
			is.inputLength = 30;
			b.addAll("Name:|name|Run Class:|startClass|Arguments:|args|Classpath:|classPaths|VM Options:|vmArgs");
		b.close();
		if (options == ADD_NEW && !selectOnTop) ed.addButton(ed.addField(new Button("Select Eve/Class File"),"select"));
	}else if (options == ZAURUS_INSTALL){
			b.addAll("Name:|name|Run Class:|startClass|Arguments:|args|VM Options:|vmArgs|Location:|location|Copy to Location:|copyOver");
			b.add("Category:","category",new Choice(mString.split("Applications|Eve|Games|Settings|Jeode"),0));
		b.close();
	}
}

//===================================================================
public void fieldChanged(String field,Editor ed)
//===================================================================
{
	if (field.equals("install")){
		if (selectInstall(install)) ed.toControls();
	}
	super.fieldChanged(field,ed);
}

//===================================================================
public void action(String field,Editor ed)
//===================================================================
{
	if (field.equals("select")){
		ed.exit(100);
	}else
		super.action(field,ed);
}
//===================================================================
public boolean selectInstall(int index)
//===================================================================
{
	Property p = (Property)all.get(index);
	if (p == null) return false;
	PropertyList pl = (PropertyList)p.value;
	File f = File.getNewFile(target);
	String fn = f.getName();
	fn = fn.substring(0,fn.length()-4);
	name = pl.getString("Name",fn);
	location = pl.getString("Location","/home/root/"+fn);
	copyOver = (location.length() != 0);
	category = pl.getString("Category","Applications");
	startClass = pl.getString("Class","");
	vmArgs = pl.getString("VMOptions","");
	args = pl.getString("Arguments","");
	return true;
}
//===================================================================
public boolean makeFrom(String target,Frame parent)
//===================================================================
{
	copyOver = true;
	if (target.endsWith(".class")){
		try{
			File f = File.getNewFile(target);
			ClassFile cf = new ClassFile(f);
			File p = cf.getClassPathFrom(f);
			classPaths = p.toString();
			if (name == null || name.trim().length() == 0)
				name = mString.leftOf(f.getFileExt(),'.');
			//programDir = p.toString();
			this.startClass = cf.className();
			return true;
		}catch(Exception e){
			new MessageBox("Class Error","That class is not valid.",MessageBox.MBOK).execute();
			return false;
		}
		/*
		StringBuffer programDir = new StringBuffer();
		String cls = UIBuilder.askClassName(target,programDir,parent);
		if (cls == null) return false;
		this.programDir = programDir.toString();
		//this.target = cls;
		this.startClass = cls;
		return true;
		*/
	}else if (target.endsWith(".eve")||target.endsWith(".zip")){
		try{
			ProgressBarForm.display(null,"Reading file...",parent);
			this.target = target;
			all = load();
			if (all != null){
				selectInstall(0);
			}
		}finally{
			ProgressBarForm.clear();
		}
		return true;
	}else
		return false;
}

public RemoteAppEntry toRemoteAppEntry(boolean forApplet)
{
	RemoteAppEntry re = new RemoteAppEntry(name,null);
	re.vmOptions = vmArgs;
	re.arguments = args;
	if (startClass == null || startClass.trim().length() == 0){
		FileStore fs = getTarget();
		if (fs == null) return null;
		try{
			FileStoreEntry fe = fs.findEntry("META-INF/EveStartClass.txt");
			if (fe == null) return null;
			InputStream in = fs.openInputStream(fe);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			re.startClass = br.readLine().trim();
			br.close();
		}catch(Exception e){
			return null;
		}finally{
			try{
				fs.close();
			}catch(Exception e2){}
		}
	}else
		re.startClass = startClass;
	File f = File.getNewFile(target);
	if (f.canRead()){
		if (!forApplet){
			if (target.endsWith(".eve")) re.eveFile = f;
			else if (target.endsWith(".zip")) re.zipFile = f;
		}else{
			re.directories.add(getTarget());
		}
	}else{
		String cp = classPaths.trim();
		String[] all = null;
		if (cp.length() != 0){
			int idx = cp.indexOf(';');
			if (idx != -1) all = mString.split(classPaths,';');
			else if (File.pathSeparatorChar != ';') all = mString.split(classPaths,File.pathSeparatorChar);
			else all = new String[]{classPaths};
		}
		if (all != null) 
			for (int i = 0; i<all.length; i++)
				re.directories.add(File.getNewFile(all[i]));
	}
	return re;
}
//===================================================================
public boolean run()
//===================================================================
{
	classPaths = classPaths.trim();
	VMOptions vo = VMOptions.getVMOptions();
	String a = target;
	if (target.endsWith(".eve") || target.endsWith(".zip")) a = "\""+a+"\"";
	if (!vo.useJavaVM && vo.shouldUsePathToEve()){
		File dir = File.getNewFile(vo.vmOptions.pathToEve).getParentFile();
		File cld = dir.getChild("eve.eve"); 
		if (cld.exists())
			a += " \""+cld.getAbsolutePath()+"\"";
	}
	if (target.endsWith(".zip")){
		if (!vo.useJavaVM) {
			a = "-cp "+a; 
			if (startClass.length() != 0) a = a+" "+startClass;
		}else{
			a = startClass;
		}
	}else{
		if (startClass.length() != 0) 
			a = a +" "+startClass;//startClass+" "+a;
		if (!vo.useJavaVM && classPaths.length() != 0)
			a = "-cp \""+classPaths+"\" "+a; 
	}
	a += " "+args;
	if (vmArgs.length() != 0) a = vmArgs+" "+a;
	if (programDir.length() != 0)
		a = "/d \""+programDir+"\" "+a;
	try{
		String[] path = Device.getVmCommandLine(a);
		if (vo.useJavaVM){
			mVector p = new mVector(path);
			p.removeElementAt(0);
			File f = File.getNewFile(vo.eveProgramDirectory).getChild("eve.jar");
			p.insertElementAt("Eve",0);
			String look = f.getAbsolutePath();
			if (target.endsWith(".zip"))
				look += File.pathSeparator+target;
			if (classPaths.length() != 0)
				look += File.pathSeparator+classPaths;
			if (File.pathSeparatorChar == ';')
				p.insertElementAt("\""+look+"\"",0);
			else
				p.insertElementAt(look,0);
			p.insertElementAt("-cp",0);
			p.insertElementAt(File.toSystemDependantPath(vo.javaVM),0);
			path = new String[p.size()];
			p.copyInto(path);
		}
		//System.out.println(new mVector(path));
		Runtime.getRuntime().exec(path);
	}catch(IOException e){
		new ReportException(e).execute();
		return false;
	}
	if (vo.exitAfterLaunch) System.exit(0);
	return true;
}

//===================================================================
public PropertyList getInstallProperties(FileStore store)
//===================================================================
{
		PropertyList pl = new PropertyList();
		try{
			InputStream in = store.openInputStream(store.findEntry("_install_.txt"));
			pl.readConfigFile(new BufferedReader(new InputStreamReader(in)));
			in.close();
			return pl;
		}catch(Exception e){
			pl = new PropertyList();
			PropertyList pl2 = new PropertyList();
			pl.set("default",pl2);
			return pl;
		}finally{
		}
}
private FileStore getTarget() 
{
	if (!target.endsWith(".eve") && !target.endsWith(".zip")) return null;
	try{
		RandomStream s = File.getNewFile(target).toRandomStream("r");
		return target.endsWith(".eve") ? (FileStore)new EveFile(s,target) : (FileStore)new ZipFile(s,target);
	}catch(Exception e){
		return null;
	}
}
/**
* Load up the icon for the shortcut.
**/
//===================================================================
public PropertyList load()
//===================================================================
{
	if (target.endsWith(".eve")||target.endsWith(".zip")){
		try{
			RandomStream s = new FileRandomStream(File.getNewFile(target),"r");
			FileStore store = target.endsWith(".eve") ? (FileStore)new EveFile(s,target) : (FileStore)new ZipFile(s,target);
			try{
				all = getInstallProperties(store);
				if (install > all.size()) install = 0;
				PropertyList pl = (PropertyList)((Property)all.get(install)).value;
				String icon = pl.getString("Icon",null);
				if (icon != null){
					if (!icon.endsWith(".png")) icon += ".png";
					InputStream in = store.openInputStream(store.findEntry(icon));
					if (in != null){
						try{
							image = new Picture(new FormattedDataSource().set(in),0);
							in.close();
							// Image is OK.
							File dir = VMApps.getImageDirectory();
							File imf = File.getNewFile("");
							for (int i = 0;;i++){
								String nm = "icon"+i+".png";
								imf.set(dir,nm);
								if (!imf.exists()) break;
							}
							in = store.openInputStream(store.findEntry(icon));
							OutputStream out = imf.toWritableStream(false);
							StreamUtils.transfer(null,in,out);
							in.close();
							out.close();
							imageName = imf.getFileExt();
						}catch(Exception e){
							//e.printStackTrace();
							//Bad image, ignore it.
						}
					}
				}
				return all;
			}catch(Exception e){
				return null;
			}finally{
				s.close();
				store.close();
			}
		}catch(IOException e){
			return null;
		}
	}else {
	}
		return null;
}
//##################################################################
}
//##################################################################

