/*
 * Created on May 8, 2006
 *
 * Michael L Brereton - www.ewesoft.com
 * 
 * 
 */
package eve.ui.print;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Vector;

import eve.fx.Color;
import eve.fx.DrawnIcon;
import eve.fx.IImage;
import eve.fx.Picture;
import eve.io.File;
import eve.sys.ImageData;
import eve.ui.Button;
import eve.ui.Gui;
import eve.ui.Label;
import eve.ui.data.Editor;
import eve.util.Comparer;
import eve.util.FormattedDataSource;
import eve.util.Utils;
import eve.zipfile.ZipEntry;
import eve.zipfile.ZipFile;

/**
 * @author Michael L Brereton
 * This Class previews images that have been generated by a eve.fx.print.PrintToImages class.
 * 
 */
//####################################################
public class PreviewViewer extends Editor {

	public int numPages;
	public String viewing = "Page 1 of 1";
	ScaledImageViewer siv = new ScaledImageViewer();
	protected Button addButton(String name, String action)
	{
		return addButton(name,action,(IImage)null);
	}
	protected Button addButton(String name, String action, IImage image)
	{
		Button b = new Button(name);
		if (image != null) Gui.iconize(b,image,true,null);
		if (buttons == null) buttons = new Vector();
		buttons.add(b);
		addField(b,action);
		return b;
	}
	/**
	 * 
	 */
	public PreviewViewer() {
		setPreferredSize(640,480);
		Label l;
		addLast(addField(l = new Label(" "),"viewing")).setCell(HSTRETCH);
		l.anchor = CENTER;
		addLast(siv.getScrollablePanel());
		setPage(0);
		addButton("<< Prev","previous");
		addButton("Next >>","next");
		addButton("Whole Page","whole");
	}

	Vector images;
	Vector entries;
	ZipFile zf;
	ImageData last;
	int curIndex = 0;
	boolean persistent;
	
	public void close(int v)
	{
		super.close(v);
		if (zf != null) try{
			zf.close();
		}catch(IOException e){}
		if (!persistent && last != null){
			last.freeImage();
		}
	}
	public void whole_action(Editor ed)
	{
		siv.fit();
	}
	public void next_action(Editor ed)
	{
		if (curIndex < numPages-1){
			setPage(++curIndex);
		}
	}
	public void previous_action(Editor ed)
	{
		if (curIndex > 0){
			setPage(--curIndex);
		}
	}
	protected ImageData getPage(int index)
	{
		if (images != null){
			persistent = true;
			return (ImageData)images.get(index);
		}else if (zf != null){
			persistent = false;
			ZipEntry ze = (ZipEntry)entries.get(index);
			try{
				InputStream in = zf.getInputStream(ze);
				FormattedDataSource fds = new FormattedDataSource().set(in);
				Picture p = new Picture(fds,0);
				in.close();
				return p;
			}catch(IOException e){
				throw new RuntimeException(e);
			}
		}
		return null;
	}
	public void setPage(int index)
	{
		if (last != null && !persistent) last.freeImage();
		last = null;
		try{
			last = getPage(index);
			siv.setImage(last);
		}catch(Exception e){
			siv.setImage(new DrawnIcon(DrawnIcon.CROSS,400,400,Color.Black));
		}
		viewing = "Page "+(index+1)+" of "+numPages;
		toControls("viewing");
	}
	private void imagesSet()
	{
		curIndex = 0;
		if (numPages == 0) {
			viewing = "No Pages";
			toControls("viewing");
		}
		else setPage(0); 
	}
	public void setImages(Vector v)
	{
		images = v;
		numPages = v.size();
		persistent = true;
		imagesSet();
	}
	public void setImages(File f)
	{
		try{
			zf = new ZipFile(f);
			Vector v = new Vector();
			for (Enumeration e = zf.entries(); e.hasMoreElements();)
				v.add(e.nextElement());
			Utils.sort(null,v,new Comparer(){
				public int compare(Object one, Object two){
					return ((ZipEntry)one).getName().compareTo(((ZipEntry)two).getName());
				}
			},false);
			entries = v;
			numPages = v.size();
			persistent = false;
			imagesSet();
		}catch(Exception e){
			e.printStackTrace();
			setImages(new Vector());
		}
	}
}

//####################################################
