
package eve.ui.table.data;
import java.util.Vector;

import eve.data.MultiListSelect;
import eve.fx.Insets;
import eve.io.File;
import eve.io.FileChooserParameters;
import eve.sys.FieldTransfer;
import eve.sys.Handle;
import eve.sys.Task;
import eve.sys.TimeOut;
import eve.sys.Vm;
import eve.ui.Button;
import eve.ui.ButtonBar;
import eve.ui.CellPanel;
import eve.ui.Control;
import eve.ui.Gui;
import eve.ui.Input;
import eve.ui.InputStack;
import eve.ui.MessageBox;
import eve.ui.ProgressAndControl;
import eve.ui.ReportException;
import eve.ui.TabbedPanel;
import eve.ui.data.Editor;
import eve.ui.data.EditorEvent;
import eve.ui.FileInput;
import eve.ui.data.LiveObject;
import eve.ui.data.UIBuilder;

/**
* This is used by a FieldTableModel to export table data to the clipboard or to a file. It
* als provides an Editor which the user can use to specify particular details about the
* export. You can execute  <b>ewe ewe.data.FieldTableExportSpecs</b> to see what it looks like.
**/
//##################################################################
public class FieldTableExportSpecs extends LiveObject{
//##################################################################

public static String lastExport = "TableData.txt";

public boolean columnHeaders = true;
public boolean rowHeaders = true;
public boolean reorderRows = true;
public boolean selectedOnly = false;
public boolean entireTable = true;

public boolean toClipboard = true;
public boolean toFile = false;
public boolean useTab = true;
public boolean useSpace = false;
public boolean useOther = false;
public String fieldSeparator = "";
public String fileName = "myFile";
public File outputFile = Vm.newFileObject().getNew(lastExport);
public MultiListSelect.SingleListSelect columns;
public boolean appendFile = false;

FieldTableModel model;
//===================================================================
public FieldTableExportSpecs() {this(new FieldTableModel());}
//===================================================================

//===================================================================
public FieldTableExportSpecs(FieldTableModel model)
//===================================================================
{
	this.model = model;
}

//===================================================================
public void addToPanel(CellPanel c,Editor f,int which)
//===================================================================
{
	f.title = "Export Table Data";
	boolean doWide = !Gui.screenIs(Gui.LONG_SCREEN) && Gui.screenIs(Gui.WIDE_SCREEN);
	TabbedPanel tp = new TabbedPanel();
	tp.modifyCardOptions(0,tp.OPTION_AUTO_SCROLL);
	c.addLast(tp);
 	CellPanel cp = new CellPanel(); cp.borderWidth = 2; cp.borderStyle = cp.BDR_NOBORDER;// mInput.inputEdge;
	CellPanel cp2 = new CellPanel();
	cp.defaultTags.set(cp.TAG_INSETS,new Insets(1,2,1,2));
	InputStack is;
	is = new InputStack(); is.setText("Export"); is.columns = 2;
	UIBuilder.addChecks(is,f,"Entire Table|entireTable|Selected Rows|selectedOnly",true,true);
	cp2.addLast(is).setCell(cp.HSTRETCH);
	is = new InputStack(); is.setText("Field Separator"); is.columns = 2;
	UIBuilder.addChecks(is,f,"Tabs|useTab|Spaces|useSpace|Other|useOther",true,true);
	is.add(f.addField(new Input(),"fieldSeparator"),null);
	cp2.addLast(is).setCell(cp.HSTRETCH);
	cp.addNext(cp2).setCell(cp.HSTRETCH);
	if (!doWide) cp.endRow();
	cp2 = new CellPanel();
	is = new InputStack(); is.setText("Destination");
	UIBuilder.addChecks(is,f,"Export to clipboard|toClipboard|Export to file|toFile",true,true);
	InputStack is2 = new InputStack(); is2.borderStyle = is2.BDR_NOBORDER;
	FileInput mf = new FileInput();
	mf.title = "Select Export Output File";
	FileChooserParameters fcp = mf.fileChooserParameters;
	fcp.add(fcp.DEFAULT_EXTENSION,"txt");
	fcp.set(fcp.TYPE,fcp.TYPE_SAVE);
	fcp.add(fcp.FILE_MASK,"*.txt - Text files");
	fcp.add(fcp.FILE_MASK,"*.* - All files");
	fcp.set(fcp.TITLE,"Select Export Output");
	/*
	mf.defaultExtension = "txt";
	mf.masks.add("*.txt - Text files");
	mf.masks.add("*.* - All files");
	mf.fileChooserOptions = ewe.filechooser.FileChooser.SAVE;
	*/
	mf.select.setToolTip("Select Export Output File");
	is2.add(f.addField(mf,"outputFile"),"File:");
	is.add(is2,null);
	cp2.addLast(is).setCell(cp.HSTRETCH);
	cp.addLast(cp2);
	tp.addItem(cp,"Export",null);
	if (model.allColumns != null){
		cp = new CellPanel();
		cp.addNext(model.getColumnSelector(columns = (MultiListSelect.SingleListSelect)model.allColumns.getCopy()));
		if (!doWide) cp.endRow();
		is = new InputStack(); is.borderWidth = 3; is.borderStyle = is.EDGE_ETCHED; is.setText("Export");
		UIBuilder.addChecks(is,f,"Column Headers|columnHeaders|Row Headers|rowHeaders|Re-order Rows|reorderRows",false,true);
		cp.addLast(is).setCell(cp.HSTRETCH);
		model.allColumns.setSelected((Vector)model.displayFields.clone());
		tp.addItem(cp,"Data",null);
	}
	ProgressAndControl pc = new ProgressAndControl();
	ButtonBar bb = new ButtonBar(); bb.borderWidth = 3; bb.borderStyle = bb.EDGE_ETCHED;
	bb.addNext(f.addField(new Button("Export").setHotKey(0,'x'),"export"));
	bb.addNext(f.cancel = new Button("Cancel").setHotKey(0,'c'));
	pc.controls.addLast(bb);
	c.addLast(pc).setCell(cp.HSTRETCH);
	f.getProperties().set("progress",pc);
	//f.resizable = true;
}

//===================================================================
public void fieldEvent(FieldTransfer ft,Editor ed,Object event)
//===================================================================
{
	if (ft.fieldName.equals("_editor_") && (event instanceof EditorEvent)){
		EditorEvent ede = (EditorEvent)event;
		if (ede.affects("toFile")) ed.modifyFields("outputFile",!toFile,ed.Disabled,0,true);
		if (ede.affects("rowHeaders")) ed.modifyFields("reorderRows",!rowHeaders,ed.Disabled,0,true);
	}
}
//===================================================================
public void fieldChanged(String fieldName,Editor ed)
//===================================================================
{
	if (fieldName.equals("useTab") || fieldName.equals("useSpace")){
		fieldSeparator = "";
		ed.toControls("fieldSeparator");
	}else if (fieldName.equals("fieldSeparator")){
		useTab = useSpace = false;
		useOther = true;
		ed.toControls("useTab,useSpace,useOther");
	}else if (fieldName.equals("useOther")){
		if (useOther){
			Gui.takeFocus((Control)ed.findFieldTransfer("fieldSeparator").dataInterface,0);
		}
	}else if (fieldName.equals("rowHeaders")){
		if (!rowHeaders) {
			reorderRows = false;
			ed.toControls("reorderRows");
		}
	}
}
//===================================================================
public void action(String fieldName,Editor ed)
//===================================================================
{
	if (fieldName.equals("export")){
		if (toFile) lastExport = outputFile.getFullPath();
		final Handle h = 
			//new samples.progress.DataTransfer()
			model.getExportTask(this)
			.start();
		final ProgressAndControl pc = (ProgressAndControl)ed.getProperties().getValue("progress",null);
		pc.startTask(h,"Exporting Data");
		new Task(){
			protected void doRun(){
				if (h.waitOnSuccess(TimeOut.Forever))
					new MessageBox("Export Complete","Data exported successfully.",MessageBox.MBOK).execute();
				else{
					if (((h.check() & h.Aborted) == 0) && h.error != null)
						new ReportException(h.error).execute();
				}
				pc.endTask();
			}
		}.start();
	}
}
//##################################################################
}
//##################################################################

