package eve.util;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import eve.data.Property;
import eve.data.PropertyList;
import eve.sys.Convert;
import eve.sys.Reflection;
import eve.sys.Vm;
import eve.sys.Wrapper;

//##################################################################
public class TextEncoder{
//##################################################################
public char [] buffer = new char[0];
public int cur = 0;
public int entries = 0;

private static TextEncoder encoder = new TextEncoder();

//===================================================================
public void clear()
//===================================================================
{
	cur = 0;
	entries = 0;
}
//-------------------------------------------------------------------
protected int expand()
//-------------------------------------------------------------------
{
	int bl = buffer.length;		
	char [] nb = new char[bl+1000];
	System.arraycopy(buffer,0,nb,0,bl);
	buffer = nb;
	bl += 1000;
	return bl;
}
public final static String hex = "0123456789ABCDEF";

/**
 * Add all keys and values.
 * Note that each key and value added are converted to Strings before being
 * added. 
 * @param map the full collection to be added.
 */
public void addValues(Map map)
{
	if (map == null) return;
	for (Iterator it = map.keySet().iterator(); it.hasNext();){
		Object k = it.next();
		String key = k.toString();
		String value = map.get(k).toString();
		addValue(key,value);
	}
}
/**
 * Add all keys and values.
 * Note that each key and value added are converted to Strings before being
 * added. 
 * @param pl the full collection to be added.
 */
public void addValues(PropertyList pl)
{
	if (pl == null) return;
	for (Iterator it = pl.iterator(); it.hasNext();){
		Property p = (Property)it.next();
		if (p == null) continue;
		if (p.value == null) continue;
		addValue(p.name,p.value.toString());
	}
}
//-------------------------------------------------------------------
protected void addValue(String name,char [] value,int start,int length)
//-------------------------------------------------------------------
{
	int d = cur;
	int bl = buffer.length;
	
	if (entries != 0) {
		if (d >= bl) bl = expand();
		buffer[d++] = '&';
	}
	char [] nc = Vm.getStringChars(name);
	int nl = nc.length;
	for (int i = 0; i<nl; i++) {
		char c = nc[i];
		if (c == ' ') c = '+';
		else if (c < ' ' || c == '+' || c == '&' || c == '%' || c == '=' || c == '|' || c == '{' || c == '}'){
			while (d+2 >= bl) bl = expand();
			buffer[d++] = '%';
			buffer[d++] = hex.charAt((c >> 4) & 0xf);
			buffer[d++] = hex.charAt(c & 0xf);
			continue;
		}
		if (d >= bl) bl = expand();
		buffer[d++] = c;
	}
	if (d >= bl) bl = expand();
	buffer[d++] = '=';
	
	int ve = start+length;
	for (int i = start; i<ve; i++) {
		char c = value[i];
		if (c == ' ') c = '+';
		else if (c < ' ' || c == '+' || c == '%' || c == '&' || c == '=' || c == '|' || c == '{' || c == '}'){
			while (d+2 >= bl) bl = expand();
			buffer[d++] = '%';
			buffer[d++] = hex.charAt((c >> 4) & 0xf);
			buffer[d++] = hex.charAt(c & 0xf);
			continue;
		}
		if (d >= bl) bl = expand();
		buffer[d++] = c;
	}
	entries++;
	cur = d;
}

//===================================================================
public void addValue(String name,String value)
//===================================================================
{
	char [] v = value.toCharArray();
	addValue(name,v,0,v.length);
}
//===================================================================
public SubString toSubString(SubString dest)
//===================================================================
{
	if (dest == null) dest = new SubString();
	dest.set(buffer,0,cur);
	return dest;
}
//===================================================================
public String toString() {return new String(buffer,0,cur);}
//===================================================================

//===================================================================
public static Object getObject(String type,String encoded,Class requestor)
//===================================================================
{
	//Vm.debug(type+" = "+encoded);
	if (type == null) return encoded;
	if (type.length() == 0)
 		if (encoded.length() != 0) return encoded;
		else return null;
	if (type.equals("java/lang/String")) return encoded;
	Class r = Reflection.forName(type.replace('/','.'),requestor);
	if (r == null) return null;
	else{
		if (r.isArray()) return arrayFromText(encoded,requestor);
		Object vobj = Reflection.newInstance(r);
		if (vobj != null){
			fromString(vobj,encoded,requestor);
		}else{
			//Vm.debug("Could not create new instance!");
		}
		return vobj;
	}
}

private static Class getForName(String name, Class requestor)
{
	if (requestor == null) return Reflection.forName(name);
	else return Reflection.forName(name,requestor);
}
//===================================================================
public static Object arrayFromText(String txt,Class requestor)
//===================================================================
{
	TextDecoder td = new TextDecoder(txt);
	String sz = td.getValue("<size>");
	if (sz == null) return null;
	int len = Convert.toInt(sz);
	String type = td.getValue("<type>");
	if (type == null) return null;
	Class ctype = getForName(type,requestor);
	Object array = Array.newInstance(ctype,len); 
	char t = type.charAt(0);
	if (t == 'L' || t == '['){
		Object [] dest = (Object [])array;
		int d = 0;
		for (int i = 0; i<td.size(); i++){
			String nm = td.getName(i);
			String v = td.getValue(i);
			if (nm.length() == 0) dest[d++] = getObject(nm,td.getValue(i),requestor);
			else if (nm.charAt(0) == '<') continue;
			else dest[d++] = getObject(nm,td.getValue(i),requestor);
		}
	}else{
		Utils.primitiveArrayFromString(array,td.getValue("<value>"));
	}
	return array;
}

/**
* This returns an encoded string as:<p>
* <className>=<objectValue><p>
* If the object is null then just "=" will be encoded.<p>
* If the class is a string then className will be empty EXCEPT if the String is empty,
* in which case the className will be "java/lang/String";
**/
//===================================================================
public static void saveFullObject(Object obj,TextEncoder te)
//===================================================================
{
	if (obj == null) te.addValue("","");
	else{
		String className = Reflection.getEncodedClassNameOnly(obj.getClass()); 
		String data = toString(obj);
		if (obj instanceof String)
			if (((String)obj).length() != 0)
				className = "";
		te.addValue(className,data);
	}
}

//===================================================================
public static Object getFullObject(String typeAndData,Class requestor)
//===================================================================
{
	//Vm.debug("typeAndData: "+typeAndData);
	TextDecoder td  = new TextDecoder(typeAndData);
	return getObject(td.getName(0),td.getValue(0),requestor);
}

public static String
//==================================================================
	toString(Object obj)
//==================================================================
{
	if (obj == null) return "";
	Class r = obj.getClass(); 
	if (r.isArray()){
		TextEncoder te = new TextEncoder();
		te.addValue("<size>",""+Array.getLength(obj));
		Class type = r.getComponentType();
		te.addValue("<type>",Reflection.getEncodedName(type));
		if (!type.isPrimitive()){
			Object [] array = (Object [])obj;
			for (int i = 0; i<array.length; i++)
				saveFullObject(array[i],te);
		}else{
			te.addValue("<value>",Utils.primitiveArrayToString(obj));
		}
		return te.toString();
	}else if (obj instanceof String) return (String)obj;
	else if (obj instanceof TextEncodable){
		TextEncodable ld = (TextEncodable)obj;
		return ld.textEncode();
/*
	}else if (isOfType(c,"mJava.PropertyList") && obj != null){
		PropertyList pl = (PropertyList)obj;
		return pl.encodeAsText();
*/
	}else if (obj instanceof Textable) {
		return ((Textable)obj).getText();
	}else if (obj instanceof Vector){
		TextEncoder te = new TextEncoder();
		Vector v = (Vector)obj;
		for (int i = 0; i<v.size(); i++)
			saveFullObject(v.get(i),te);
		return te.toString();
	}else if (obj instanceof Encodable){
		return Utils.textEncode((Encodable)obj);
	}else
		return obj.toString();
}

public static String
//==================================================================
	toString(Field f,Object o,Wrapper toUse)
//==================================================================
{
	if (toUse == null) toUse = new Wrapper();
	try{
		Reflection.getFieldValue(o,f,toUse);
		if (toUse.getType() == toUse.OBJECT)// || w.getType() == w.ARRAY)
			return toString(toUse.getObject());
		else
			return toUse.toString();
	}catch(Exception e){
		return null;
	}
}

public static void fromString(Object obj,String val)
{
	if (obj == null) return;
	fromString(obj,val,obj.getClass());
}

public static void
//==================================================================
	fromString(Object obj,String val,Class requestor)
//==================================================================
{
	if (obj == null) return;
	else if (obj instanceof TextEncodable){
		((TextEncodable)obj).textDecode(val);
		return;
/*
	}else if (isOfType(c,"mJava.PropertyList")){
		PropertyList pl = (PropertyList)obj;
		pl.decodeAsText(val);
		return;
*/
	}else if (obj instanceof Textable){
		((Textable)obj).setText(val);
		return;
	}else if (obj instanceof Vector){
		Vector v = (Vector)obj;
		v.clear();
		TextDecoder td = new TextDecoder(val);
		for (int i = 0; i<td.size(); i++){
			v.add(getObject(td.getName(i),td.getValue(i),requestor));
		}
	}else if (obj instanceof Stringable){
		((Stringable)obj).fromString(val);
		return;
	}else if (obj instanceof Encodable){
		Utils.textDecode((Encodable)obj,val);
		return;
	}
}
public static Class [] textables = {
		TextEncodable.class, Vector.class, Textable.class, Stringable.class, Encodable.class
};
	//"Leve/util/TextEncodable;","Ljava/util/Vector;","Leve/util/Textable;","Leve/util/Stringable;","Leve/util/Encodable;"


public static void
//==================================================================
	fromString(Field f,Object o,String val,Wrapper toUse)
//==================================================================
{
	try{
		if (toUse == null) toUse = new Wrapper();
		if (o == null) return;
		Class type = f.getType();
		if (!type.isPrimitive()){
			if (!type.isArray()){
				if (type.equals(String.class)){
					Reflection.setFieldValue(o,f,toUse.setObject(val));
				}else for (int i = 0; i<textables.length; i++){
					if (Reflection.isTypeOf(type,textables[i])){
						Reflection.getFieldValue(o,f,toUse);
						Object obj = toUse.getObject();
						if (obj == null){
							obj = Reflection.newInstance(type);
							if (obj != null) Reflection.setFieldValue(o,f,toUse.setObject(obj));
						}
						if (obj != null) fromString(obj,val,o.getClass());
						return;
					}
				}
			}else{
				Reflection.setFieldValue(o,f,toUse.setObject(arrayFromText(val,o.getClass())));
			}
		}else{ // Primitive
			toUse.zero(type);
			toUse.fromString(val);
			Reflection.setFieldValue(o,f,toUse);
		}
	}catch(IllegalAccessException e){
		
	}
}

//===================================================================
public static char [] toText(long value,int numBytes,char [] dest,int offset)
//===================================================================
{
	if (dest == null) {
		dest = new char[numBytes*2];
		offset = 0;
	}
	for (int i = numBytes*2-1; i >= 0; i--){
		int v = (int)(value & 0xf);
		if (v < 10) dest[offset+i] = (char)('0'+v);
		else dest[offset+i] = (char)('A'+v-10);
		value >>= 4;
	}
	return dest;
}
//===================================================================
public static char [] toText(double value, char [] dest,int offset)
//===================================================================
{
	return toText(Convert.toLongBitwise(value),8,dest,offset);
}
//===================================================================
public static long longFromText(int numBytes,char [] from,int offset)
//===================================================================
{
	if (from == null) return 0;
	long value = 0;
	for (int i = 0; i < numBytes*2; i++){
		char c = from[offset+i];
		int v = c-'0';
		if (c >= 'a' && c <= 'f') v = 10+c-'a';
		else if (c >= 'A' && c <= 'F') v = 10+c-'A';
		value <<= 4;
		value |= (v & 0xf);
	}
	return value;
}

//===================================================================
public static double doubleFromText(char [] from,int offset)
//===================================================================
{
	long got = longFromText(8,from,offset);
	return Convert.toDoubleBitwise(got);
}
/*
//===================================================================
public static String toText(String typeName,Object value)
//===================================================================
{
	char [] ret = null;
	ewe.sys.Long ln = ewe.sys.Long.l1;
	if (value instanceof ewe.sys.Double)
		ln.set(Convert.toLongBitwise(((ewe.sys.Double)value).value));
	else 
		if (value instanceof ewe.sys.Long) ln = (ewe.sys.Long)value;
		
	switch(typeName.charAt(0)){
		case 'Z':
		case 'B': ret = toText(ln.value,1,null,0); break;
		case 'C': 
		case 'S': ret = toText(ln.value,2,null,0); break;
		case 'I': ret = toText(ln.value,4,null,0); break;
		case 'D':
		case 'F': 
		case 'J': ret = toText(ln.value,8,null,0); break;
		case '[': 
		case 'L': return toString(value);
	}
	if (ret != null) return new String(ret);
	return null;
}
*/
/*
//===================================================================
public static Wrapper toWrapper(String typeName,String actualClass,String from,Class requestor)
//===================================================================
{
	return toWrapper(typeName,actualClass,Vm.getStringChars(from),0,from.length(),requestor);
}
//===================================================================
public static Wrapper toWrapper(String typeName,String actualClass,char [] from,int offset,int length,Class requestor)
//===================================================================
{
	Wrapper ret = new Wrapper();
	switch(typeName.charAt(0)){
		case 'Z': return ret.setBoolean(longFromText(1,from,offset) != 0);
		case 'B': return ret.setByte((byte)longFromText(1,from,offset));
		case 'C': return ret.setChar((char)longFromText(2,from,offset));
		case 'S': return ret.setShort((short)longFromText(2,from,offset));
		case 'I': return ret.setInt((int)longFromText(4,from,offset));
		case 'J': return ret.setLong(longFromText(8,from,offset));
		case 'D':	return ret.setDouble(doubleFromText(from,offset));
		case 'F': return ret.setFloat((float)doubleFromText(from,offset));
		case '[': return ret.setObject(arrayFromText(new String(from,offset,length),requestor));
		case 'L': {
			Class c = Reflection.forEncodedName(actualClass);
			if (c == null) return ret.setObject(null);
			else {
				String enc = new String(from,offset,length);
				if (actualClass.equals("java/lang/String")) //FIXME - L; ?
					return ret.setObject(enc);
				Object obj = Reflection.newInstance(c);
				if (obj != null)
					fromString(obj,enc,requestor);
				return ret.setObject(obj);
			}
		}
	}
	return ret;
}
*/
/*

//===================================================================
public static String toText(Wrapper value)
//===================================================================
{
	TextEncoder te = new TextEncoder();
	if (value.getType() == Wrapper.OBJECT){// || value.getType() == '['){
		Object tr = value.getObject();
		Class r = tr == null ? null : tr.getClass();
		if (r == null) te.addValue("","");
		else te.addValue(Reflection.getEncodedClassNameOnly(r),toString(tr));
	}else{
		switch(value.getType()){
			case 'Z': te.addValue("",new String(toText(value.getBoolean() ? 1 : 0,1,null,0))); break;
			case 'B': te.addValue("",new String(toText(value.getByte(),1,null,0))); break;
			case 'C': te.addValue("",new String(toText(value.getChar(),2,null,0))); break;
			case 'S': te.addValue("",new String(toText(value.getShort(),2,null,0))); break;
			case 'I': te.addValue("",new String(toText(value.getInt(),4,null,0))); break;
			case 'D': te.addValue("",new String(toText(value.getDouble(),null,0))); break;
			case 'F': te.addValue("",new String(toText(value.getFloat(),null,0))); break;
			case 'J': te.addValue("",new String(toText(value.getLong(),8,null,0))); break;
		}
	}
	return te.toString();
}
//===================================================================
public static Object fromText(String typeName,Object value0
//===================================================================
{
	char [] ret = null;
	ewe.sys.Long ln = ewe.sys.Long.l1;
	if (value instanceof ewe.sys.Double)
		ln.set(Convert.toLongBitwise(((ewe.sys.Double)value).value));
	else 
		if (value instanceof ewe.sys.Long) ln = (ewe.sys.Long)value;
		
	switch(typeName.charAt(0)){
		case 'Z':
		case 'B': ret = toText(ln.value,1,null,0); break;
		case 'C': 
		case 'S': ret = toText(ln.value,2,null,0); break;
		case 'I': ret = toText(ln.value,4,null,0); break;
		case 'D':
		case 'F':
		case 'J': ret = toText(ln.value,8,null,0); break;
		case 'L': return toString(value);
	}
}
*/
/*
//===================================================================
public String fromText(String typeName,Object value)
//===================================================================
{
	char [] ret = null;
	ewe.sys.Long ln = ewe.sys.Long.l1;
	if (value instanceof ewe.sys.Double)
		ln.set(Convert.toLongBitwise(((ewe.sys.Double)value).value));
	else 
		if (value instanceof ewe.sys.Long) ln = (ewe.sys.Long)value;
		
	switch(typeName.charAt(0)){
		case 'Z':
		case 'B': ret = toText(ln.value,1,null,0); break;
		case 'C': 
		case 'S': ret = toText(ln.value,2,null,0); break;
		case 'I': ret = toText(ln.value,4,null,0); break;
		case 'D':
		case 'F':
		case 'J': ret = toText(ln.value,8,null,0); break;
	}
	if (ret != null) return new String(ret);
	return null;
}
*/
//##################################################################
}
//##################################################################

