package eve.util;
import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.Vector;

import eve.sys.Cache;
import eve.sys.Reflection;
//import eve.sys.*;

/**
 * A vector is an array of object references. The vector grows and shrinks
 * dynamically as objects are added and removed.
 * <p>
 * Here is an example showing a vector being used:
 *
 * <pre>
 * ...
 * Vector vec = new Vector();
 * vec.add(obj1);
 * vec.add(obj2);
 * ...
 * vec.insert(3, obj3);
 * vec.del(2);
 * if (vec.getCount() > 5)
 * ...
 * </pre>
 */
public class mVector  extends java.util.Vector implements eve.data.DataUnit
{
	
//private WeakSet reclaimed;
/*
public void initializeReclaimedList(boolean useSoftReferences)
{
	reclaimed = new WeakSet(useSoftReferences);
}
*/
	private static Object[] objectBuffer = new Object[0];
	
/** Constructs an empty vector. */
//===================================================================
public mVector()
//===================================================================
{
	super();
}
/**
 * Constructs an empty vector with a given initial size. The size is
 * the initial size of the vector's internal object array. The vector
 * will grow as needed when objects are added.
 */
//===================================================================
public mVector(int size)
//===================================================================
{
	super(size);
}
//===================================================================
public mVector(int size,int incrementCount)
//===================================================================
{
	super(size,incrementCount);
}
//===================================================================
public mVector(Object [] items)
//===================================================================
{
	this(8);
	addAll(items);
}
//===================================================================
public mVector(java.util.Vector items)
//===================================================================
{
	this(8);
	addAll(items);
}
/**
 * An Object is about to be reclaimed, so clean it up as necessary.
 * @param obj the object to be reclaimed.
 * @return true to continue and place the reclaimed object back into the
 * set of reclaimed objects.
 */
/*
protected boolean reclaim(Object obj)
{
	return true;
}
*/
/**
 * Reclaim all the elements in the mVector.
 */
/*
public synchronized void reclaimAllElements()
{
	reclaimElements(0,size());
}
*/
/**
 * Reclaim a number of objects in the mVector.
 * @param startIndex the index of the first item to reclaim.
 * @param num the number of items to reclaim.
 */
/*
public synchronized void reclaimElements(int startIndex, int num)
{
	if (startIndex < 0 || startIndex+num > elementCount || num < 0) 
		throw new IndexOutOfBoundsException();
	if (num == 0) return;
	for (int i = 0; i<num; i++){
		Object obj = elementData[i+startIndex];
		if (obj == null) continue;
		reclaim(obj);
		if (reclaimed == null) reclaimed = new WeakSet(true); 
		reclaimed.add(obj);
	}
	if (num == elementCount) removeAllElements();
	else {
		System.arraycopy(elementData,startIndex+num,elementData,startIndex,num);
		if (num == 1) elementData[elementCount-num] = null;
		else Utils.zeroArrayRegion(elementData,elementCount-num,num);
		elementCount -= num;
	}
}
*/
/**
 * Add an object from an outside source to the mVector's set of reclaimed objects.
 * @param obj the object to be added. 
 */
/*
public synchronized void addToReclaimedList(Object obj)
{
	if (obj == null || !reclaim(obj)) return;
	if (reclaimed == null) reclaimed = new WeakSet(true);
	reclaimed.add(obj);
}
*/
/**
 * Return the next reclaimed object.
 */
/*
public synchronized Object getReclaimedObject()
{
	if (reclaimed == null) return null;
	return reclaimed.takeRef();
}
*/
/**
 * Get a reclaimed object for a Vector if the Vector is an mVector.
 * @param v a java.util.Vector 
 * @return an Object from the reclaimed list if the object is an mVector, or
 * null if the object is not an mVector or if there are no objects in the
 * reclaimed list.
 */
/*
public static Object getReclaimedObject(Vector v)
{
	if (!(v instanceof mVector)) return null;
	return ((mVector)v).getReclaimedObject();
}
*/
/**
 * Reclaim all the elements in the Vector assuming it is an mVector.
 * @param v a java.util.Vector.
 */
/*
public static void reclaimAllElements(Vector v)
{
	if (!(v instanceof mVector)) return;
	((mVector)v).reclaimAllElements();
}
*/
//===================================================================
public synchronized void copyFrom(Object other)
//===================================================================
{
	//super.copyFrom(other);
	if (other instanceof java.util.Vector){
		removeAllElements();
		synchronized(other){
			elementData = new Object[((java.util.Vector)other).size()];
			((java.util.Vector)other).copyInto(elementData);
			elementCount = elementData.length;
		}
	}else if (other instanceof Object []){
		removeAllElements();
		addAll((Object [])other);
	}
}
//===================================================================
public synchronized void addCopiesFrom(Object [] object,int start,int length)
//===================================================================
{
	ensureCapacity(length+elementCount);
	for (int i = 0; i<length; i++)
		elementData[elementCount+i] = Utils.getCopy(object[i+start]);
	elementCount += length;
}
//===================================================================
public synchronized void copyCopiesFrom(Object other)
//===================================================================
{
	if (other instanceof java.util.Vector){
		Object[] a = toArray((java.util.Vector)other);
		addCopiesFrom(a,0,a.length);
	}else if (other instanceof Object []){
		Object [] ot = (Object [])other;
		addCopiesFrom(ot,0,ot.length);
	}
}
/**
* This returns a Vector that is a copy of this one, where all the items
* are themselves copies of the original items.
**/
//===================================================================
public synchronized mVector getFullCopy()
//===================================================================
{
	mVector v = (mVector)getNew();
	v.copyCopiesFrom(this);
	return v;
}
/**
 * Insert an object at the given index.
 * @param index The index to insert the object.
 * @param obj The object to add.
 * @deprecated Use add(index,obj) instead.
 */
//===================================================================
public synchronized void insert(int index, Object obj)
//===================================================================
{
	insertElementAt(obj,index);	
}
/*
//===================================================================
public synchronized void add(Object obj)
//===================================================================
{
	addElement(obj);
}
*/
/** Inserts an object at the given index. */

//===================================================================
public synchronized void add(int index, Object obj)
//===================================================================
{
	insertElementAt(obj,index);
}

/** Deletes the object reference at the given index. */
//===================================================================
public synchronized void del(int index)
//===================================================================
{
	removeElementAt(index);
}
//===================================================================
public Object get(int index)
//===================================================================
{
	return elementAt(index);
}
/** Sets the object at the given index. */

//===================================================================
public synchronized void set(int index, Object obj)
//===================================================================
{
	setElementAt(obj,index);
}

/**
 * Finds the index of the given object. The list is searched using a O(n) linear
 * search through all the objects in the vector.
 */
//MLB replaced with a new version.
/*
public int find(Object obj)
	{
	for (int i = 0; i < count; i++)
		if (items[i] == obj)
			return i;
	return -1;
	}
*/
/** Returns the number of objects in the vector. */
//===================================================================
public int getCount()
//===================================================================
{
	return elementCount;
}

/** Converts the vector to an array of objects. */
//===================================================================
public synchronized Object [] toObjectArray()
//===================================================================
{
	Object objs[] = new Object[elementCount];
	copyInto(objs);
	return objs;
}
//===================================================================
public synchronized void clear()
//===================================================================
{
	removeAllElements();
}
/**
This is used by the find() method, while the equals() method is used
by contains() and indexOf().
**/
/*
//===================================================================
public int compare(Object one,Object two)
//===================================================================
{
	return Utils.compare(one,two);
}
*/
/**
 * Finds the index of the given object using the compare() method. 
 * The list is searched using a O(n) linear
 * search through all the objects in the vector using the compare method.
 */
	/*
//===================================================================
public int find(Object obj)
//===================================================================
{
	for (int i = 0; i < count; i++)
		if (compare(items[i],obj) == 0)
			return i;
	return -1;
}
*/
//===================================================================
public synchronized void addAll(Object [] objs, int start, int length)
//===================================================================
{
	if (objs == null || length < 1) return;
	ensureCapacity(elementCount+length);
	System.arraycopy(objs,start,elementData,elementCount,length);
	elementCount += length;
}
//===================================================================
public synchronized void addAll(Object [] objs)
//===================================================================
{
	if (objs == null) return;
	else addAll(objs,0,objs.length);
}
//===================================================================
public synchronized void addAll(java.util.Vector objs)
//===================================================================
{
	if (objs == null) return;
	addAll(toArray(objs));
}
/**
* The destination array MUST be big enough to hold the vector.
**/
//===================================================================
public synchronized void copyInto(Object [] array,int destPos)
//===================================================================
{
	System.arraycopy(elementData,0,array,destPos,elementCount);
	//for (int i = 0; i<count; i++) array[i+destPos] = items[i];	
}
/**
* The destination array MUST be big enough to hold the vector.
**/
//===================================================================
public void copyInto(Object [] array) {copyInto(array,0);}
//===================================================================

//===================================================================
public Object find(ObjectFinder finder)
//===================================================================
{
	for (int i = 0;; i++){
		Object check = null;
		synchronized(this){
			if (i >= elementCount) return null;
			check = elementData[i];
		}
		if (finder.lookingFor(check)) 
			return check;
	}
}
/**
* This adds an object to the end of the Vector v. If v is null a new Vector is created
* and returned.
**/
//===================================================================
public static java.util.Vector add(java.util.Vector v,Object toAdd)
//===================================================================
{
	if (v == null) v = new mVector();
	v.addElement(toAdd);
	return v;
}
/**
* This inserts an object to the front of the Vector v. If v is null a new Vector is created
* and returned. This is not meant to be a high-performance Stack implementation.
**/
//===================================================================
public static java.util.Vector push(java.util.Vector v,Object toPush)
//===================================================================
{
	if (v == null) v = new Vector();
	v.insertElementAt(toPush,0);
	return v;
}
/**
* This removes and returns the object at the front of the Vector v. If v is null OR v is 
* empty, it will return null. This is not meant to be a high-performance Stack implementation.
**/
//===================================================================
public static Object pop(java.util.Vector v)
//===================================================================
{
	if (v == null) return null;
	synchronized(v){
		if (v.size() == 0) return null;
		Object ret = v.elementAt(0);
		v.removeElementAt(0);
		return ret;
	}
}
/**
* This returns an iterator for the specified Vector. If the vector is null it will return
* an Iterator with no elements.
**/
//===================================================================
public static Iterator iterator(java.util.Vector v)
//===================================================================
{
	return v == null ? new ObjectIterator(null) : v.iterator();
}
/**
 * Copy the vector into the destination array, creating a new one if the provided destination is not
	big enough to hold all the elements. If the provided destination array is big enough for the vector,
	the elements are copied into the array starting from the first index. If the array is bigger than the
	size of the vector, the extra elements of the array are set to null. If the array is smaller than the
	vector, then a new array is created, whose runtime type is the same as the destination parameter, and
	whose size is the same size as the vector. The vector elements are then copied into the newly created
	array and this array is then returned.
 * @param [] dest A destination object array to copy the vector elements into.
 * @return The destination object array if it is big enough, or a new array if it is not.
 */
//===================================================================
public synchronized Object [] toArray(Object [] dest)
//===================================================================
{
	if (dest == null) throw new NullPointerException();
	if (dest.length >= elementCount) {
		copyInto(dest);
		for (int i = elementCount; i<dest.length; i++) dest[i] = null;
	}else{
		Class c = dest.getClass();
		Class cp = c.getComponentType();
		dest = (Object[])Array.newInstance(cp,elementCount);
		copyInto(dest);
	}
	return dest;
}
/**
* Create a new Object array and copy all the elements into it.
* @return A new Object array containing the vector's elements.
*/
//===================================================================
public Object [] toArray()
//===================================================================
{
	return toArray(new Object[0]);
}
//===================================================================
public static Object[] toArray(java.util.Vector source)
//===================================================================
{
	if (source == null) return new Object[0];
	synchronized(source){
		int size = source.size();
		Object[] ret = new Object[size];
		source.copyInto(ret);
		return ret;
	}
}
//===================================================================
public Object getNew()
//===================================================================
{
	return Reflection.newInstance(getClass());
	//return new mVector();
}

//===================================================================
public Object getCopy()
//===================================================================
{
	mVector v = (mVector)getNew();
	v.copyFrom(this);
	return v;
}
/**
This just compares the sizes of the two vectors.
**/
//===================================================================
public int compareTo(Object other)
//===================================================================
{
	if (!(other instanceof java.util.Vector)) return 1;
	return elementCount - ((java.util.Vector)other).size();
}
public static Vector addAll(Vector dest, Object[] data)
{
	if (data != null && dest != null) synchronized(dest){
		dest.ensureCapacity(dest.size()+data.length);
		for (int i = 0; i<data.length; i++)
			dest.add(data[i]);
	}
	return dest;
}
public static Vector newVector(Object[] data)
{
	return addAll(new Vector(),data);
}
//===================================================================
public synchronized String toString()
//===================================================================
{
	String out = "[";
	for (int i = 0; i<size(); i++){
		if (i != 0) out+=", ";
		out += eve.util.mString.toString(elementAt(i));
	}
	out += "]";
	return out;
}
/**
 * Remove elements from the vector and cache them.
 * @return
 */
public synchronized static Vector cacheAllElements(Vector v)
{
	int sz = 0;
	synchronized(v){
		sz = v.size();
		if (objectBuffer == null || objectBuffer.length < sz)
			objectBuffer = new Object[sz];
		v.copyInto(objectBuffer);
		v.removeAllElements();
	}
	//
	for (int i = 0; i<sz; i++)
		Cache.put(objectBuffer[i]);
	//
	return v;
}

}

